@@ -1,8 +1,10 @@
-#!/usr/bin/env python2.5
+#!/usr/bin/env python
+from __future__ import print_function
 from __future__ import absolute_import
 import numpy as N
 import sys
 import time
+from six.moves import xrange
 
 # c: aa.cc
 
@@ -55,6 +57,6 @@ for i in xrange(niter):
     a -= lr * N.sum(g_hidin, axis=0)
 
 total_time = time.time() - t
-print 'time: ',total_time, 'err: ', err
-print ' of which', dot_time, 'was spent on dot. Fraction:', dot_time / total_time
+print('time: ',total_time, 'err: ', err)
+print(' of which', dot_time, 'was spent on dot. Fraction:', dot_time / total_time)
 
