@@ -1,3 +1,4 @@
+from __future__ import print_function
 import sys, timeit
 import numpy
 import scikits.image.opencv
@@ -7,7 +8,7 @@ try:
     ker_shape =  int(sys.argv[3]), int(sys.argv[4])
     dtype = sys.argv[5]
 except:
-    print >> sys.stderr, "Usage: %s <img rows> <img cols> <ker rows> <ker cols> <dtype> [nb_call]" % sys.argv[0]
+    print("Usage: %s <img rows> <img cols> <ker rows> <ker cols> <dtype> [nb_call]" % sys.argv[0], file=sys.stderr)
     sys.exit(-1)
 
 nb_call = 1
@@ -27,5 +28,5 @@ def f():
     scikits.image.opencv.cvFilter2D(img, ker)
 """)
 time = T.repeat(repeat=3, number=nb_call)
-print min(time), "opencv"
+print(min(time), "opencv")
 
