@@ -1,5 +1,7 @@
+from __future__ import print_function
 import theano
 import theano.tensor as tt
+from six.moves import xrange
 
 k = tt.iscalar("k")
 A = tt.vector("A")
@@ -20,5 +22,5 @@ power = theano.function(inputs=[A, k],
                         outputs=final_result,
                         updates=updates)
 
-print power(range(10), 2)
+print(power(list(range(10)), 2))
 #[  0.   1.   4.   9.  16.  25.  36.  49.  64.  81.]
