@@ -6,6 +6,7 @@
 # 1. Raw results
 
 
+from __future__ import print_function
 import numpy
 import theano
 import theano.tensor as tt
@@ -62,24 +63,24 @@ predict = theano.function(inputs=[], outputs=prediction,
 
 if any([x.op.__class__.__name__ in ['Gemv', 'CGemv', 'Gemm', 'CGemm'] for x in
 train.maker.fgraph.toposort()]):
-    print 'Used the cpu'
+    print('Used the cpu')
 elif any([x.op.__class__.__name__ in ['GpuGemm', 'GpuGemv'] for x in
 train.maker.fgraph.toposort()]):
-    print 'Used the gpu'
+    print('Used the gpu')
 else:
-    print 'ERROR, not able to tell if theano used the cpu or the gpu'
-    print train.maker.fgraph.toposort()
+    print('ERROR, not able to tell if theano used the cpu or the gpu')
+    print(train.maker.fgraph.toposort())
 
 for i in range(training_steps):
     pred, err = train()
 #print "Final model:"
 #print w.get_value(), b.get_value()
 
-print "target values for D"
-print D[1]
+print("target values for D")
+print(D[1])
 
-print "prediction on D"
-print predict()
+print("prediction on D")
+print(predict())
 
 """
 
