@@ -1,7 +1,9 @@
 import theano
-from theano.compile.mode import Mode
+from theano.compile.mode import Mode, AddFeatureOptimizer
+from theano.gof.toolbox import NoOutputFromInplace
 import theano.tensor as T
 
+
 def test_no_output_from_implace():
 
     x = T.matrix()
@@ -11,15 +13,20 @@ def test_no_output_from_implace():
 
     # Ensure that the elemwise op that produces the output is inplace when
     # using a mode that does not include the optimization
-    fct_no_opt = theano.function([x,y], b, mode="FAST_RUN")
+    fct_no_opt = theano.function([x, y], b, mode="FAST_RUN")
     op = fct_no_opt.maker.fgraph.outputs[0].owner.op
     assert (hasattr(op, 'destroy_map') and 0 in op.destroy_map)
 
     # Ensure that the elemwise op that produces the output is not inplace when
     # using a mode that includes the optimization
-    mode_opt = Mode(linker="cvm", optimizer="fast_run")
-    mode_opt = mode_opt.including("add_no_output_from_inplace")
+    opt = AddFeatureOptimizer(NoOutputFromInplace())
+    mode_opt = Mode(linker="cvm", optimizer="fast_run").register((opt, 49.9))
 
-    fct_opt = theano.function([x,y], b, mode=mode_opt)
+    fct_opt = theano.function([x, y], b, mode=mode_opt)
     op = fct_opt.maker.fgraph.outputs[0].owner.op
     assert (not hasattr(op, 'destroy_map') or 0 not in op.destroy_map)
+
+
+def test_including():
+    mode = theano.Mode(optimizer='merge')
+    mode.including('fast_compile')
