@@ -7,8 +7,10 @@ import theano
 from theano.tensor import dmatrix, iscalar, lscalar, dmatrices
 from theano import tensor
 
-from theano.compile.sharedvalue import *
-from theano.compile.pfunc import *
+from theano.compile import In
+from theano.compile import pfunc
+from theano.compile import shared
+from theano.compile import config
 
 
 def data_of(s):
@@ -25,7 +27,7 @@ class Test_pfunc(unittest.TestCase):
         a = lscalar()
         b = shared(1)
         f1 = pfunc([a], (a + b))
-        f2 = pfunc([Param(a, default=44)], a + b, updates={b: b + 1})
+        f2 = pfunc([In(a, value=44)], a + b, updates={b: b + 1})
         self.assertTrue(b.get_value() == 1)
         self.assertTrue(f1(3) == 4)
         self.assertTrue(f2(3) == 4)
@@ -77,7 +79,7 @@ class Test_pfunc(unittest.TestCase):
         try:
             pfunc([w], theano.tensor.sum(w * w))
             assert False
-        except TypeError, e:
+        except TypeError as e:
             msg = 'Cannot use a shared variable (w) as explicit input'
             if str(e).find(msg) < 0:
                 raise
@@ -112,13 +114,13 @@ class Test_pfunc(unittest.TestCase):
         b = shared(7)
         out = a + b
 
-        f = pfunc([Param(a, strict=False)], [out])
+        f = pfunc([In(a, strict=False)], [out])
         # works, rand generates float64 by default
         f(numpy.random.rand(8))
         # works, casting is allowed
         f(numpy.array([1, 2, 3, 4], dtype='int32'))
 
-        f = pfunc([Param(a, strict=True)], [out])
+        f = pfunc([In(a, strict=True)], [out])
         try:
             # fails, f expects float64
             f(numpy.array([1, 2, 3, 4], dtype='int32'))
@@ -130,14 +132,14 @@ class Test_pfunc(unittest.TestCase):
         a_out = a * 2  # assuming the op which makes this "in place" triggers
 
         # using mutable=True will let fip change the value in aval
-        fip = pfunc([Param(a, mutable=True)], [a_out], mode='FAST_RUN')
+        fip = pfunc([In(a, mutable=True)], [a_out], mode='FAST_RUN')
         aval = numpy.random.rand(10)
         aval2 = aval.copy()
         assert numpy.all(fip(aval) == (aval2 * 2))
         assert not numpy.all(aval == aval2)
 
         # using mutable=False should leave the input untouched
-        f = pfunc([Param(a, mutable=False)], [a_out], mode='FAST_RUN')
+        f = pfunc([In(a, mutable=False)], [a_out], mode='FAST_RUN')
         aval = numpy.random.rand(10)
         aval2 = aval.copy()
         assert numpy.all(f(aval) == (aval2 * 2))
@@ -182,9 +184,9 @@ class Test_pfunc(unittest.TestCase):
         a = tensor.wvector('a')  # int16
         b = tensor.bvector('b')  # int8
         c = tensor.bscalar('c')  # int8
-        f = pfunc([Param(a, allow_downcast=True),
-                   Param(b, allow_downcast=False),
-                   Param(c, allow_downcast=None)],
+        f = pfunc([In(a, allow_downcast=True),
+                   In(b, allow_downcast=False),
+                   In(c, allow_downcast=None)],
                   (a + b + c))
 
         # Both values are in range. Since they're not ndarrays (but lists),
@@ -195,7 +197,7 @@ class Test_pfunc(unittest.TestCase):
         # For performance reasons, no check of the data is explicitly performed
         # (It might be OK to change this in the future.)
         self.assertRaises(TypeError, f,
-                [3], numpy.array([6], dtype='int16'), 1)
+                          [3], numpy.array([6], dtype='int16'), 1)
 
         # Value too big for a, silently ignored
         assert numpy.all(f([2 ** 20], numpy.ones(1, dtype='int8'), 1) == 2)
@@ -211,9 +213,9 @@ class Test_pfunc(unittest.TestCase):
         b = tensor.fscalar('b')
         c = tensor.fscalar('c')
 
-        f = pfunc([Param(a, allow_downcast=True),
-                   Param(b, allow_downcast=False),
-                   Param(c, allow_downcast=None)],
+        f = pfunc([In(a, allow_downcast=True),
+                   In(b, allow_downcast=False),
+                   In(c, allow_downcast=None)],
                   (a + b + c))
 
         # If the values can be accurately represented, everything is OK
@@ -236,9 +238,9 @@ class Test_pfunc(unittest.TestCase):
         b = tensor.fvector('b')
         c = tensor.fvector('c')
 
-        f = pfunc([Param(a, allow_downcast=True),
-                   Param(b, allow_downcast=False),
-                   Param(c, allow_downcast=None)],
+        f = pfunc([In(a, allow_downcast=True),
+                   In(b, allow_downcast=False),
+                   In(c, allow_downcast=None)],
                   (a + b + c))
 
         # If the values can be accurately represented, everything is OK
@@ -274,7 +276,7 @@ class Test_pfunc(unittest.TestCase):
         # For performance reasons, no check of the data is explicitly performed
         # (It might be OK to change this in the future.)
         self.assertRaises(TypeError, g,
-                [3], numpy.array([6], dtype='int16'), 0)
+                          [3], numpy.array([6], dtype='int16'), 0)
 
         # Value too big for b, raises TypeError
         self.assertRaises(TypeError, g, [3], [312], 0)
@@ -283,7 +285,7 @@ class Test_pfunc(unittest.TestCase):
         # Everything here should behave like with False
         assert numpy.all(h([3], [6], 0) == 9)
         self.assertRaises(TypeError, h,
-                [3], numpy.array([6], dtype='int16'), 0)
+                          [3], numpy.array([6], dtype='int16'), 0)
         self.assertRaises(TypeError, h, [3], [312], 0)
 
     def test_allow_downcast_floatX(self):
@@ -343,13 +345,13 @@ class Test_pfunc(unittest.TestCase):
         # the update_var has type matrix, and the update expression
         # is a broadcasted scalar, and that should be allowed.
         self.assertRaises(TypeError, theano.function, inputs=[], outputs=[],
-                updates={output_var: output_var.sum().dimshuffle('x', 'x')})
+                          updates={output_var: output_var.sum().dimshuffle('x', 'x')})
 
     def test_duplicate_updates(self):
         x, y = dmatrices('x', 'y')
         z = shared(numpy.ones((2, 3)))
         self.assertRaises(ValueError, theano.function, [x, y], [z],
-                updates=[(z, (z + x + y)), (z, (z - x))])
+                          updates=[(z, (z + x + y)), (z, (z - x))])
 
     def test_givens(self):
         x = shared(0)
@@ -365,7 +367,7 @@ class Test_pfunc(unittest.TestCase):
         z = tensor.ivector()
         c = z * y
         f = pfunc([y], (c + 7),
-                givens={z: theano._asarray([4, 4, 4], dtype='int32')})
+                  givens={z: theano._asarray([4, 4, 4], dtype='int32')})
         assert numpy.all(f([1, 1, 1]) == [11, 11, 11])
         assert x.get_value() == 0
 
@@ -432,7 +434,7 @@ class Test_pfunc(unittest.TestCase):
         self.assertRaises(TypeError, pfunc, [], [x], no_default_updates=(x))
         self.assertRaises(TypeError, pfunc, [], [x], no_default_updates=x)
         self.assertRaises(TypeError, pfunc, [], [x],
-                no_default_updates='canard')
+                          no_default_updates='canard')
 
         # Mix explicit updates and no_default_updates
         g1 = pfunc([], [x], updates=[(x, (x - 1))], no_default_updates=True)
@@ -548,7 +550,7 @@ class Test_pfunc(unittest.TestCase):
     def test_default_updates_input(self):
         x = shared(0)
         y = shared(1)
-        if theano.gof.python_int_bitwidth() == 32:
+        if theano.configdefaults.python_int_bitwidth() == 32:
             a = iscalar('a')
         else:
             a = lscalar('a')
@@ -616,7 +618,7 @@ class Test_pfunc(unittest.TestCase):
     def test_duplicate_inputs(self):
         x = theano.tensor.lscalar('x')
         self.assertRaises(theano.compile.UnusedInputError,
-                theano.function, [x, x, x], x)
+                          theano.function, [x, x, x], x)
 
     def test_update_same(self):
         # There was a bug in CVM, triggered when a shared variable
@@ -698,12 +700,12 @@ class Test_aliasing_rules(unittest.TestCase):
 
         # rule #2 reading back from theano-managed memory
         assert not numpy.may_share_memory(A.get_value(borrow=False),
-                data_of(A))
+                                          data_of(A))
 
     def test_sparse_input_aliasing_affecting_inplace_operations(self):
         ##
-        ## Note this test will never fail because I am not aware of any
-        ## inplace op on sparse variables
+        # Note this test will never fail because I am not aware of any
+        # inplace op on sparse variables
         try:
             import scipy.sparse as sp
         except ImportError:
@@ -711,12 +713,12 @@ class Test_aliasing_rules(unittest.TestCase):
             pass
 
         from theano.sparse import enable_sparse
-        if enable_sparse == False:
+        if not enable_sparse:
             raise SkipTest('Optional package sparse disabled')
 
         from theano import sparse
 
-        ## Note: to trigger this bug with theano rev 4586:2bc6fc7f218b,
+        # Note: to trigger this bug with theano rev 4586:2bc6fc7f218b,
         #        you need to make in inputs mutable (so that inplace
         #        operations are used) and to break the elemwise composition
         #        with some non-elemwise op (here dot)
@@ -726,7 +728,7 @@ class Test_aliasing_rules(unittest.TestCase):
         f = theano.function([theano.In(x, mutable=True),
                              theano.In(y, mutable=True)],
                             (x + y) + (x + y))
-        ## Test 1. If the same variable is given twice
+        # Test 1. If the same variable is given twice
 
         # Compute bogus values
         m = sp.csc_matrix(numpy.asarray(
@@ -752,7 +754,7 @@ class Test_aliasing_rules(unittest.TestCase):
 
     def test_input_aliasing_affecting_inplace_operations(self):
 
-        ## Note: to trigger this bug with theano rev 4586:2bc6fc7f218b,
+        # Note: to trigger this bug with theano rev 4586:2bc6fc7f218b,
         #        you need to make in inputs mutable (so that inplace
         #        operations are used) and to break the elemwise composition
         #        with some non-elemwise op (here dot)
@@ -760,12 +762,12 @@ class Test_aliasing_rules(unittest.TestCase):
         y = theano.tensor.dvector()
         m1 = theano.tensor.dmatrix()
         m2 = theano.tensor.dmatrix()
-        f = theano.function([theano.In(x,  mutable=True),
-                             theano.In(y,  mutable=True),
+        f = theano.function([theano.In(x, mutable=True),
+                             theano.In(y, mutable=True),
                              theano.In(m1, mutable=True),
                              theano.In(m2, mutable=True)],
                             theano.dot((x * 2), m1) + theano.dot((y * 3), m2))
-        ## Test 1. If the same variable is given twice
+        # Test 1. If the same variable is given twice
 
         # Compute bogus values
         v = numpy.asarray([1, 2, 3, 4, 5], dtype='float64')
@@ -792,7 +794,7 @@ class Test_aliasing_rules(unittest.TestCase):
 
     def test_partial_input_aliasing_affecting_inplace_operations(self):
 
-        ## Note: to trigger this bug with theano rev 4586:2bc6fc7f218b,
+        # Note: to trigger this bug with theano rev 4586:2bc6fc7f218b,
         #        you need to make in inputs mutable ( so that inplace
         #        operations are used) and to break the elemwise composition
         #        with some non-elemwise op ( here dot )
@@ -803,21 +805,20 @@ class Test_aliasing_rules(unittest.TestCase):
         m2 = theano.tensor.dmatrix()
         m3 = theano.tensor.dmatrix()
 
-        ## Test 2. If variables only partial overlap
+        # Test 2. If variables only partial overlap
         #   more exactly we care about the case when we have a,b,c
         #   and a shares memory with b, b shares memory with c, but
         #   c does not share memory with a
 
         f = theano.function(
-                [theano.In(x,  mutable=True),
-                 theano.In(y,  mutable=True),
-                 theano.In(z,  mutable=True),
-                 theano.In(m1, mutable=True),
-                 theano.In(m2, mutable=True),
-                 theano.In(m3, mutable=True)],
-                (theano.dot((x * 2), m1)
-                    + theano.dot((y * 3), m2)
-                    + theano.dot((z * 4), m3)))
+            [theano.In(x, mutable=True),
+             theano.In(y, mutable=True),
+             theano.In(z, mutable=True),
+             theano.In(m1, mutable=True),
+             theano.In(m2, mutable=True),
+             theano.In(m3, mutable=True)],
+            (theano.dot((x * 2), m1) + theano.dot((y * 3), m2) +
+             theano.dot((z * 4), m3)))
 
         # Compute bogus values
         v = numpy.asarray([1, 2, 3, 4, 5], dtype='float64')
@@ -874,14 +875,14 @@ class Test_aliasing_rules(unittest.TestCase):
         assert not numpy.may_share_memory(R, data_of(A))
 
         f = pfunc([D], (DD * 4),
-                updates=[(A, (DD[:1] * 3)), (B, (DD[:1] * 2))])
+                  updates=[(A, (DD[:1] * 3)), (B, (DD[:1] * 2))])
         R = f(C)
         assert not numpy.may_share_memory(data_of(A), data_of(B))
         assert not numpy.may_share_memory(R, data_of(B))
         assert not numpy.may_share_memory(R, data_of(A))
 
         f = pfunc([D], (DD * 4),
-                updates=[(A, (DD[:1] * 3)), (B, (DD[:1] * 3))])
+                  updates=[(A, (DD[:1] * 3)), (B, (DD[:1] * 3))])
         R = f(C)
         assert not numpy.may_share_memory(data_of(A), data_of(B))
         assert not numpy.may_share_memory(R, data_of(B))
@@ -950,7 +951,7 @@ class Test_aliasing_rules(unittest.TestCase):
         data_of_b = data_of(B)
 
         f = pfunc([], [], updates=[(A, B[:, ::-1]), (B, A.T)])
-        #theano.printing.debugprint(f)
+        # theano.printing.debugprint(f)
         f()
         # correctness (doesn't actually test the view...)
         assert numpy.all(data_of(A) == -.5)
