@@ -1,59 +1,115 @@
 """
 VMs that run Theano graph computations.
+
 A VM is not actually different from a Linker, we just decided
 VM was a better name at some point.
+
 """
-import link
+from . import link
+from collections import defaultdict
 import logging
 import os
 import sys
 import time
 import warnings
 
-from theano.compat.python2x import all
-
-from theano.configparser import (config, AddConfigVar,
-                                 BoolParam, ConfigParam, _config_var_list)
+from theano.configparser import (config, _config_var_list)
 
 import theano.gof.cmodule
 
+from six import iteritems, itervalues
+from six.moves import xrange
+
 logger = logging.getLogger(__name__)
 
-AddConfigVar('profile',
-             "If VM should collect profile information",
-             BoolParam(False),
-             in_c_key=False)
-AddConfigVar('profile_optimizer',
-             "If VM should collect optimizer profile information",
-             BoolParam(False),
-             in_c_key=False)
-AddConfigVar('profile_memory',
-             "If VM should collect memory profile information and print it",
-             BoolParam(False),
-             in_c_key=False)
 
+def calculate_reallocate_info(order, fgraph, storage_map, compute_map_re,
+                              dependencies):
+    reallocated_info = {}
+    viewed_by = {}
+    for var in fgraph.variables:
+        viewed_by[var] = []
+    view_of = {}
+    pre_allocated = set([])
+    allocated = set([])
 
-def filter_vm_lazy(val):
-    if val == 'False' or val is False:
-        return False
-    elif val == 'True' or val is True:
-        return True
-    elif val == 'None' or val is None:
-        return None
-    else:
-        raise ValueError('Valid values for an vm.lazy parameter '
-                         'should be None, False or True, not `%s`.' % val)
+    for idx in range(len(order)):
+        node = order[idx]
+        dmap = getattr(node.op, 'destroy_map', None)
+        vmap = getattr(node.op, 'view_map', None)
+
+        idx_o = 0
+        for out in node.outputs:
+            for var in node.outputs:
+                compute_map_re[var][0] = 1
+            ins = None
+            if dmap and idx_o in dmap:
+                idx_v = dmap[idx_o]
+                assert len(idx_v) == 1, ("Here we only support the possibility"
+                                         " to destroy one input")
+                ins = node.inputs[idx_v[0]]
+            if vmap and idx_o in vmap:
+                assert ins is None
+                idx_v = vmap[idx_o]
+                assert len(idx_v) == 1, ("Here we only support the possibility"
+                                         " to view one input")
+                ins = node.inputs[idx_v[0]]
+            if ins is not None:
+                assert isinstance(ins, theano.Variable)
+                origin = view_of.get(ins, ins)
+                view_of[out] = origin
+                viewed_by[origin].append(out)
+            idx_o += 1
+
+        for ins in node.inputs:
+            assert not (ins in view_of and viewed_by[ins])
+            if (getattr(ins, 'ndim', None) == 0 and not storage_map[ins][0] and
+                    ins not in fgraph.outputs and ins.owner and
+                    all([compute_map_re[v][0]
+                         for v in dependencies.get(ins, [])]) and
+                    ins not in allocated):
+                # Constant Memory cannot be changed
+                # Constant and shared variables' storage_map value is not empty
+                reuse_out = None
+                if ins not in view_of and not viewed_by.get(ins, []):
+                    # where gc
+                    for i in range(idx + 1, len(order)):
+                        if reuse_out is not None:
+                            break
+                        for out in order[i].outputs:
+                            if (getattr(out, 'ndim', None) == 0 and
+                                    out not in pre_allocated and
+                                    ins.type == out.type):
+                                reuse_out = out
+                                pre_allocated.add(out)
+                                allocated.add(ins)
+                                break
+                elif ins in view_of:
+                    origin = view_of[ins]
+                    if ins in viewed_by[origin]:
+                        viewed_by[origin].remove(ins)
+                    if (not viewed_by[origin] and
+                            origin not in fgraph.inputs and
+                            not isinstance(origin, theano.Constant)):
+                        # where gc
+                        for i in range(idx + 1, len(order)):
+                            if reuse_out is not None:
+                                break
+                            for out in order[i].outputs:
+                                if (getattr(out, 'ndim', None) == 0 and
+                                        out not in pre_allocated and
+                                        ins.type == out.type):
+                                    reuse_out = out
+                                    pre_allocated.add(out)
+                                    allocated.add(ins)
+                                    break
+                if reuse_out is not None:
+                    reallocated_info[ins] = [ins, reuse_out]
+
+    return reallocated_info
 
-AddConfigVar('vm.lazy',
-             "Useful only for the vm linkers. When lazy is None,"
-             " auto detect if lazy evaluation is needed and use the apropriate"
-             " version. If lazy is True/False, force the version used between"
-             " Loop/LoopGC and Stack.",
-             ConfigParam('None', filter_vm_lazy),
-             in_c_key=False)
 
 class VM(object):
-
     """
     A VM object's __call__ method evaluates a Theano program.
 
@@ -66,33 +122,35 @@ class VM(object):
     advantage of lazy computation, though they still produce the correct
     output for lazy nodes.
 
-    Attributes:
+    Parameters
+    ----------
+    nodes
+        A list of nodes in toposort order.
+    thunks
+        A list of thunks to execute those nodes, in toposort order.
+    pre_call_clear
+        A list of containers to empty at the beginning of each call.
 
-    call_counts - list of integers, one for each thunk. call_count[i] is the
-        number of times thunks[i] was called in the course of computations
-        performed by call_with_timers().
+    Attributes
+    ----------
+    call_counts
+        List of integers, one for each thunk. call_count[i] is the number of
+        times thunks[i] was called in the course of computations performed by
+        call_with_timers().
+    call_times
+        List of floats, one for each thunk. call_times[i] is the amount of
+        runtime spent on thunks[i] in the course of computations performed by
+        call_with_timers().
 
-    call_times - list of floats, one for each thunk. call_times[i] is
-        the amount of runtime spent on thunks[i] in the course of
-        computations performed by call_with_timers().
-
-    need_update_inputs - bool. True indicates that Function.__call__
-        must implement the feedback from output storage to input
-        storage. False means it *must not* repeat that feedback.
+    need_update_inputs : bool
+        True indicates that Function.__call__ must implement the feedback from
+        output storage to input storage. False means it *must not* repeat that
+        feedback.
 
     """
 
     def __init__(self, nodes, thunks, pre_call_clear):
-        """
-        Allocate a virtual machine.
 
-        nodes - a list of nodes in toposort order
-
-        thunks - a list of thunks to execute those nodes, in toposort order
-
-        pre_call_clear - a list of containers to empty at the beginning of each
-                         call.
-        """
         if len(nodes) != len(thunks):
             raise ValueError()
         self.nodes = nodes
@@ -113,6 +171,7 @@ class VM(object):
 
         Postcondition - all output variables have been computed.  VMs vary in
         what exactly this means and how it is done.
+
         """
         raise NotImplementedError('override me')
 
@@ -123,6 +182,7 @@ class VM(object):
         Free internal variables and outputs.  Essentially, free as much memory
         as possible without intefering with the ability to evaluate subsequent
         calls.
+
         """
         raise NotImplementedError('override me')
 
@@ -149,7 +209,7 @@ class VM(object):
             profile.node_cleared_order = self.node_cleared_order[:]
 
         if hasattr(self, 'dependencies'):
-            profile.dependencies = self.dependencies.copy()
+            profile.dependencies = self.dependencies
 
         # clear the timer info out of the buffers
         for i in xrange(len(self.call_times)):
@@ -158,11 +218,13 @@ class VM(object):
 
 
 class Loop(VM):
-
     """
     Unconditional start-to-finish program execution in Python.
     No garbage collection is allowed on intermediate results.
+
     """
+    # Some other part of Theano query that information
+    allow_gc = False
 
     def __call__(self):
         if self.time_thunks:
@@ -189,15 +251,17 @@ class Loop(VM):
 
 
 class LoopGC(VM):
-
     """
     Unconditional start-to-finish program execution in Python.
     Garbage collection is possible on intermediate results.
+
     """
 
     def __init__(self, nodes, thunks, pre_call_clear, post_thunk_clear):
         super(LoopGC, self).__init__(nodes, thunks, pre_call_clear)
         self.post_thunk_clear = post_thunk_clear
+        # Some other part of Theano query that information
+        self.allow_gc = True
         if not (len(nodes) == len(thunks) == len(post_thunk_clear)):
             raise ValueError()
 
@@ -234,7 +298,6 @@ class LoopGC(VM):
 
 
 class Stack(VM):
-
     """
     Finish-to-start evalution order of thunks.
 
@@ -288,8 +351,8 @@ class Stack(VM):
             # destroy_dependencies
             # --------------------
             # The destroy_dependencies is a list of variables that are implicit
-            # dependencies induced by a destroy_map (compare node.inputs which
-            # are *explicit* dependencies). The variables in
+            # dependencies induced by destroy_map and view_map (compared to
+            # node.inputs which are *explicit* dependencies). The variables in
             # destroy_dependencies would be impossible to compute after the
             # current `node` runs, because node.thunk() is going to destroy a
             # common input variable needed by whatever node owns each variable
@@ -306,9 +369,11 @@ class Stack(VM):
             raise ValueError("Must set dependencies when using GC")
 
     def run_thunk_of_node(self, node):
-        """Run the thunk corresponding to Apply instance `node`
+        """
+        Run the thunk corresponding to Apply instance `node`.
 
         Calls self.callback if it is defined.
+
         """
         idx = self.node_idx[node]
         t0 = time.time()
@@ -345,7 +410,7 @@ class Stack(VM):
         last_apply_stack_len = -1
 
         # This record all function inputs/shared varibles and constants
-        for var, data in self.storage_map.iteritems():
+        for var, data in iteritems(self.storage_map):
             if data[0] is None:
                 continue
             if hasattr(var.type, 'get_shape_info'):
@@ -422,9 +487,10 @@ class Stack(VM):
                                     st = "c"
                                 self.variable_strides[var] = st
                     except Exception:
-                        link.raise_with_op(current_apply,
-                                           self.thunks[self.node_idx[current_apply]],
-                                           storage_map=storage_map)
+                        link.raise_with_op(
+                            current_apply,
+                            self.thunks[self.node_idx[current_apply]],
+                            storage_map=storage_map)
                     for o in current_apply.outputs:
                         compute_map[o][0] = 1
 
@@ -435,9 +501,9 @@ class Stack(VM):
                         for i in current_apply.inputs:
                             # Garbage Collection -> check if anybody else uses
                             # this input
-                            if (dependencies[i]
-                                    and i.owner
-                                    and i not in self.outputs):
+                            if (dependencies[i] and
+                                    i.owner and
+                                    i not in self.outputs):
                                 if all(compute_map[v][0]
                                         for v in dependencies[i]):
                                     storage_map[i][0] = None
@@ -447,7 +513,7 @@ class Stack(VM):
                                     # DO NOT set compute_map to 0
 
                                     # If values become False and the
-                                    #current_apply is still in the
+                                    # current_apply is still in the
                                     # stack, this will cause it to be
                                     # recomputed! This can cause wrong value
                                     # with some combination of inplace op.
@@ -458,10 +524,13 @@ class Stack(VM):
                                                 'destroy_map',
                                                 False)):
                                         warnings.warn(
-                                            "There was a bug that existed in the default Theano configuration,"
-                                            " only in the development version between July 5th 2012"
-                                            " and July 30th 2012. This was not in a released version."
-                                            " The bug was affecting this script.",
+                                            "There was a bug that existed in "
+                                            "the default Theano configuration,"
+                                            " only in the development version "
+                                            "between July 5th 2012 and "
+                                            "July 30th 2012. This was not in "
+                                            "a released version. The bug was "
+                                            "affecting this script.",
                                             # The stack level is not good when
                                             # inside a Scan.
                                             stacklevel=3
@@ -492,9 +561,10 @@ class Stack(VM):
                     self.call_times[current_idx] += dt
 
                 except Exception:
-                    link.raise_with_op(current_apply,
-                                       self.thunks[self.node_idx[current_apply]],
-                                       storage_map)
+                    link.raise_with_op(
+                        current_apply,
+                        self.thunks[self.node_idx[current_apply]],
+                        storage_map=storage_map)
 
                 if requires:
                     for r in requires:
@@ -553,7 +623,7 @@ class Stack(VM):
 
         if self.allow_gc:
             for v in storage_map:
-                if v.owner and not v in self.outputs:
+                if v.owner and v not in self.outputs:
                     if compute_map[v][0] == 2:
                         continue
                     else:
@@ -565,7 +635,7 @@ class Stack(VM):
 
 
 try:
-    import lazylinker_c
+    from . import lazylinker_c
 
     class CVM(lazylinker_c.CLazyLinker, VM):
 
@@ -574,7 +644,7 @@ try:
             # skip VM.__init__
 except ImportError:
     pass
-except (OSError, theano.gof.cmodule.MissingGXX), e:
+except (OSError, theano.gof.cmodule.MissingGXX) as e:
     # OSError happens when g++ is not installed.  In that case, we
     # already changed the default linker to something else then CVM.
     # Currently this is the py linker.
@@ -585,32 +655,36 @@ except (OSError, theano.gof.cmodule.MissingGXX), e:
 
 
 class VM_Linker(link.LocalLinker):
-
     """
     Class that satisfies the Linker interface by acting as a VM factory.
+
+    Parameters
+    ----------
+    allow_gc
+        Force the virtual machine to clean up unnecessary
+        references, in order to allow garbage collection on
+        intermediate values during computation of a function.
+        If None use as default the value of the Theano flag allow_gc.
+    use_cloop
+        Use the C-based virtual machine if possible
+    callback
+        A callable object to call after each call to a thunk within
+        the virtual machine.  It will be called with four arguments called
+        'node', 'thunk', 'storage_map', and 'compute_map'.
+    lazy
+        Useful only when use_cloop is False. When lazy is None, use the
+        theano flag vm.lazy value. Then if we have a None (default) we auto
+        detect if lazy evaluation is needed and use the apropriate
+        version. If lazy is True or False, we force the version used
+        between Loop/LoopGC and Stack.
+    c_thunks
+        If None or True, don't change the default. If False,
+        don't compile c code for the thunks.
+
     """
 
     def __init__(self, allow_gc=None, use_cloop=False, callback=None,
-                 lazy=None, schedule=None):
-        """
-        allow_gc - force the virtual machine to clean up unnecessary
-            references, in order to allow garbage collection on
-            intermediate values during computation of a function.
-            If None use as default the value of the Theano flag allow_gc.
-
-        use_cloop - use the C-based virtual machine if possible
-
-        callback - a callable object to call after each call to a thunk within
-            the virtual machine.  It will be called with four arguments called
-            'node', 'thunk', 'storage_map', and 'compute_map'.
-
-        lazy - Useful only when use_cloop is False. When lazy is None, use the
-            theano flag vm.lazy value. Then if we have a None (default) we auto
-            detect if lazy evaluation is needed and use the apropriate
-            version. If lazy is True or False, we force the version used
-            between Loop/LoopGC and Stack.
-
-        """
+                 lazy=None, schedule=None, c_thunks=None):
         # Note: if more parameters are added to __init__, make sure to forward
         # them in the "type(self)(...)" call in the "accept" method below.
         if allow_gc is None:
@@ -620,19 +694,27 @@ class VM_Linker(link.LocalLinker):
         self.use_cloop = use_cloop
         self.callback = callback
         self.lazy = lazy
+        self.c_thunks = c_thunks
         self.updated_vars = {}
         if schedule:
             self.schedule = schedule
 
     def accept(self, fgraph, no_recycling=None):
         """
-        :param fgraph: a PerformLinker can have accepted one FunctionGraph
-            instance at a time.
 
-        :param no_recycling: WRITEME
+        Parameters
+        ----------
+        fgraph
+            A PerformLinker can have accepted one FunctionGraph instance
+            at a time.
+        no_recycling
+            WRITEME
+
+        Returns
+        -------
+        Self if fgraph is the first FunctionGraph that has ever been
+        associated to self, else, a new VM_Linker associated to fgraph.
 
-        :returns: self if fgraph is the first FunctionGraph that has ever been
-            associated to self, else, a new VM_Linker associated to fgraph.
         """
         if (config.profile and
                 hasattr(theano, 'sandbox') and
@@ -658,7 +740,8 @@ class VM_Linker(link.LocalLinker):
                 use_cloop=self.use_cloop,
                 callback=self.callback,
                 lazy=self.lazy,
-                schedule=self.schedule
+                schedule=self.schedule,
+                c_thunks=self.c_thunks,
             ).accept(fgraph, no_recycling)
         self.fgraph = fgraph
         self.no_recycling = no_recycling
@@ -677,18 +760,21 @@ class VM_Linker(link.LocalLinker):
         Returns dict: variable K -> list of variables [v1, v2, v3, ...]
         for each K in variables.
 
-
         The variables v1, v2, ... are the full set of variables that depend
         directly on K. When we know that none of them will need to be
         computed, we know that:
-        * K will not need to be computed
-        * if K is already computed, it can be released for garbage collection
-
+        * K will not need to be computed.
+        * If K is already computed, it can be released for garbage collection.
 
         Parameters
         ----------
-        variables - iterable over the variables used in a graph computation.
+        variables
+            Iterable over the variables used in a graph computation.
 
+        Notes
+        -----
+        It doesn't take care of the view_map/destroy_map. So it means it relies
+        on Python gc no to free the object real storage.
 
         N.B. gc means garbage collection
 
@@ -701,6 +787,9 @@ class VM_Linker(link.LocalLinker):
             # way of getting it back.
             #
             # XXX if k has no clients... what is it doing in the computation?
+            # Fred guess: it could happen for node with multiple outputs when
+            # we don't use all outputs.
+
             if k.owner and k.clients:
                 ls = []
                 for cl in k.clients:
@@ -714,7 +803,7 @@ class VM_Linker(link.LocalLinker):
                 post_thunk_clear,
                 computed,
                 compute_map,
-                updated_vars
+                updated_vars,
                 ):
 
         pre_call_clear = [storage_map[v] for v in self.no_recycling]
@@ -727,7 +816,7 @@ class VM_Linker(link.LocalLinker):
             if self.use_cloop and config.profile_memory:
                 warnings.warn(
                     'CVM does not support memory profile, using Stack VM.')
-            # Needed when allow_gc=True and profiling
+            # Needed for allow_gc=True, profiling and storage_map reuse
             deps = self.compute_gc_dependencies(storage_map)
             vm = Stack(
                 nodes, thunks, pre_call_clear,
@@ -748,13 +837,12 @@ class VM_Linker(link.LocalLinker):
 
             nodes_idx_inv = {}
             vars_idx_inv = {}
-            for (node, i) in nodes_idx.items():
+            for (node, i) in iteritems(nodes_idx):
                 nodes_idx_inv[i] = node
-            for (var, i) in vars_idx.items():
+            for (var, i) in iteritems(vars_idx):
                 vars_idx_inv[i] = var
 
             # put storage_map and compute_map into a int-based scheme
-            n_applies = len(nodes)
             storage_map_list = [storage_map[vars_idx_inv[i]]
                                 for i in xrange(len(vars_idx_inv))]
             compute_map_list = [compute_map[vars_idx_inv[i]]
@@ -763,7 +851,7 @@ class VM_Linker(link.LocalLinker):
                 assert type(storage_map_list[0]) is list
                 assert type(compute_map_list[0]) is list
 
-            # Needed when allow_gc=True and profiling
+            # Needed for allow_gc=True, profiling and storage_map reuse
             dependency_map = self.compute_gc_dependencies(storage_map)
             dependency_map_list = [
                 [vars_idx[d] for d in dependency_map[vars_idx_inv[i]]]
@@ -787,7 +875,7 @@ class VM_Linker(link.LocalLinker):
 
             # build the var owner array
             var_owner = [None] * len(vars_idx)
-            for (var, i) in vars_idx.items():
+            for (var, i) in iteritems(vars_idx):
                 if var.owner:
                     var_owner[i] = nodes_idx[var.owner]
 
@@ -815,7 +903,7 @@ class VM_Linker(link.LocalLinker):
             # values of the update expressions).
             update_storage = []
             update_in_from_out = {}
-            for (ivar, ovar) in updated_vars.items():
+            for (ivar, ovar) in iteritems(updated_vars):
                 update_in_from_out[vars_idx[ovar]] = vars_idx[ivar]
             for oidx in output_vars:
                 if oidx in update_in_from_out:
@@ -858,12 +946,14 @@ class VM_Linker(link.LocalLinker):
                         nodes,
                         thunks,
                         pre_call_clear,
-                        post_thunk_clear)
+                        post_thunk_clear,
+                    )
                 else:
                     vm = Loop(
                         nodes,
                         thunks,
-                        pre_call_clear)
+                        pre_call_clear,
+                    )
             else:
                 # Needed when allow_gc=True and profiling
                 deps = self.compute_gc_dependencies(storage_map)
@@ -876,21 +966,37 @@ class VM_Linker(link.LocalLinker):
         return vm
 
     def make_all(self, profiler=None, input_storage=None,
-                 output_storage=None,
+                 output_storage=None, storage_map=None,
                  ):
         fgraph = self.fgraph
         order = self.schedule(fgraph)
         no_recycling = self.no_recycling
 
         input_storage, output_storage, storage_map = link.map_storage(
-            fgraph, order, input_storage, output_storage)
+            fgraph, order, input_storage, output_storage, storage_map)
         compute_map = {}
         for k in storage_map:
             compute_map[k] = [k.owner is None]
 
         thunks = []
+
+        # Collect Reallocation Info
+        compute_map_re = defaultdict(lambda: [0])
+        for var in fgraph.inputs:
+            compute_map_re[var][0] = 1
+
+        if getattr(fgraph.profile, 'dependencies', None):
+            dependencies = getattr(fgraph.profile, 'dependencies')
+        else:
+            dependencies = self.compute_gc_dependencies(storage_map)
+
+        reallocated_info = calculate_reallocate_info(
+            order, fgraph, storage_map, compute_map_re, dependencies)
+
         for node in order:
             try:
+                if self.c_thunks is False:
+                    node.op._op_use_c_code = False
                 thunks.append(node.op.make_thunk(node,
                                                  storage_map,
                                                  compute_map,
@@ -900,7 +1006,7 @@ class VM_Linker(link.LocalLinker):
                     # So if they didn't specify that its lazy or not, it isn't.
                     # If this member isn't present, it will crash later.
                     thunks[-1].lazy = False
-            except Exception, e:
+            except Exception as e:
                 e.args = ("The following error happened while"
                           " compiling the node", node, "\n") + e.args
                 raise
@@ -908,15 +1014,26 @@ class VM_Linker(link.LocalLinker):
             thunk.inputs = [storage_map[v] for v in node.inputs]
             thunk.outputs = [storage_map[v] for v in node.outputs]
 
+        lazy = self.lazy
+        if lazy is None:
+            lazy = config.vm.lazy
+        if lazy is None:
+            lazy = not all([(not th.lazy) for th in thunks])
+        if not (lazy or (config.profile and config.profile_memory) or
+                self.use_cloop or self.callback):
+            for pair in itervalues(reallocated_info):
+                storage_map[pair[1]] = storage_map[pair[0]]
+
         computed, last_user = link.gc_helper(order)
         if self.allow_gc:
             post_thunk_clear = []
             for node in order:
                 clear_after_this_thunk = []
                 for input in node.inputs:
-                    if ((input in computed)
-                            and (input not in fgraph.outputs)
-                            and (node == last_user[input])):
+                    if (input in computed and
+                            input not in fgraph.outputs and
+                            node == last_user[input] and
+                            input not in reallocated_info):
                         clear_after_this_thunk.append(storage_map[input])
                 post_thunk_clear.append(clear_after_this_thunk)
         else:
@@ -927,7 +1044,7 @@ class VM_Linker(link.LocalLinker):
                           post_thunk_clear,
                           computed,
                           compute_map,
-                          self.updated_vars
+                          self.updated_vars,
                           )
 
         vm.storage_map = storage_map
@@ -939,3 +1056,8 @@ class VM_Linker(link.LocalLinker):
                  for output, storage in zip(fgraph.outputs, output_storage)],
                 thunks,
                 order)
+
+    def __setstate__(self, d):
+        self.__dict__.update(d)
+        if not hasattr(self, 'c_thunks'):
+            self.c_thunks = True
