@@ -1,31 +1,34 @@
 """Driver for gradient calculations."""
+from __future__ import print_function
+import six.moves.builtins as builtins
+import logging
+import time
+import traceback
+import warnings
 
+import numpy  # for numeric_grad
+from six import itervalues
+from six.moves import StringIO
+
+import theano
+
+from theano import gof
+from theano.gof import Variable
+from theano.compat import OrderedDict, izip
+from six.moves import xrange, reduce
+from theano.gof.null_type import NullType, null_type
+from theano.gof.op import get_debug_values
+from theano.compile import ViewOp
+
+np = numpy
 __authors__ = "James Bergstra, Razvan Pascanu, Arnaud Bergeron, Ian Goodfellow"
 __copyright__ = "(c) 2011, Universite de Montreal"
 __license__ = "3-clause BSD License"
 __contact__ = "theano-dev <theano-dev@googlegroups.com>"
 
 __docformat__ = "restructuredtext en"
-
-import __builtin__
-from itertools import izip
-import logging
-import time
-import warnings
 _logger = logging.getLogger('theano.gradient')
 
-import numpy  # for numeric_grad
-np = numpy
-
-import theano
-
-from theano import gof
-from theano.gof import Variable
-from theano.compat.python2x import OrderedDict
-from theano.gof.null_type import NullType, null_type
-from theano.gof.op import get_debug_values
-from theano.compile import ViewOp
-
 # we can't do "import theano.tensor"
 # tensor depends on theano.compile
 # theano.compile depends on theano.gradient (this file)
@@ -86,7 +89,7 @@ def grad_not_implemented(op, x_pos, x, comment=""):
     return (NullType((
         "This variable is Null because the grad method for "
         "input %s (%s) of the %s op is not implemented. %s"
-        ) % (x_pos, x, op, comment)))()
+    ) % (x_pos, x, op, comment)))()
 
 
 def grad_undefined(op, x_pos, x, comment=""):
@@ -283,7 +286,7 @@ def Rop(f, wrt, eval_points):
                 same_type_eval_points.append(y)
 
         seen_nodes[node] = op.R_op(node.inputs, same_type_eval_points)
-    #end _traverse
+    # end _traverse
 
     # Populate the dictionary
     for out in f:
@@ -358,7 +361,8 @@ def Lop(f, wrt, eval_points, consider_constant=None,
 
 def grad(cost, wrt, consider_constant=None,
          disconnected_inputs='raise', add_names=True,
-         known_grads=None, return_disconnected='zero'):
+         known_grads=None, return_disconnected='zero',
+         null_gradients='raise'):
     """
     Return symbolic gradients for one or more variables with respect to some
     cost.
@@ -367,52 +371,51 @@ def grad(cost, wrt, consider_constant=None,
     see :mod:`gradient`. For information on how to implement the gradient of
     a certain Op, see :func:`grad`.
 
-    :type cost: Scalar (0-dimensional) tensor variable.
-        May optionally be None if known_grads is provided.
-    :param cost: a scalar with respect to which we are differentiating
+    Parameters
+    ----------
+    cost : scalar (0-dimensional) tensor variable or None
+        Value with respect to which we are differentiating.  May be
+        `None` if known_grads is provided.
+    wrt : variable or list of variables
+        term[s] for which we want gradients
+    consider_constant : list of variables
+        expressions not to backpropagate through
+    disconnected_inputs : {'ignore', 'warn', 'raise'}
+        Defines the behaviour if some of the variables in `wrt` are
+        not part of the computational graph computing `cost` (or if
+        all links are non-differentiable). The possible values are:
 
-    :type wrt: Tensor variable or list of variables.
-    :param wrt: term[s] for which we want gradients
-
-    :type consider_constant: list of variables
-    :param consider_constant: a list of expressions not to backpropagate
-        through
-
-    :type disconnected_inputs: string
-    :param disconnected_inputs: Defines the behaviour if some of the variables
-        in ``wrt`` are not part of the computational graph computing ``cost``
-        (or if all links are non-differentiable). The possible values are:
         - 'ignore': considers that the gradient on these parameters is zero.
         - 'warn': consider the gradient zero, and print a warning.
         - 'raise': raise DisconnectedInputError.
-
-    :type add_names: bool
-    :param add_names: If True, variables generated by grad will be named
-        (d<cost.name>/d<wrt.name>) provided that both cost and wrt have
-        names
-
-    :type known_grads: dict
-    :param known_grads: If not None, a dictionary mapping variables to their
-            gradients. This is useful in the case where you know the
-            gradient on some variables but do not know the original
-            cost.
-
-    :type return_disconnected: string
-    :param return_disconnected:
+    add_names : bool
+        If True, variables generated by grad will be named
+        (d<cost.name>/d<wrt.name>) provided that both cost and wrt
+        have names
+    known_grads : dict, optional
+        A dictionary mapping variables to their gradients. This is
+        useful in the case where you know the gradient on some
+        variables but do not know the original cost.
+    return_disconnected : {'zero', 'None', 'Disconnected'}
         - 'zero' : If wrt[i] is disconnected, return value i will be
                    wrt[i].zeros_like()
         - 'None' : If wrt[i] is disconnected, return value i will be
                    None
         - 'Disconnected' : returns variables of type DisconnectedType
+    null_gradients : {'raise', 'return'}
+        Defines the behaviour if some of the variables in `wrt` have a
+        null gradient. The possibles values are:
 
-    :rtype: variable or list/tuple of Variables (matching `wrt`)
+        - 'raise' : raise a NullTypeGradError exception
+        - 'return' : return the null gradients
 
-    :return: symbolic expression of gradient of `cost` with respect to each
-             of the `wrt` terms.
-             If an element of `wrt` is not differentiable with respect
-             to the output, then a zero variable is returned.
-             It returns an object of same type as `wrt`: a list/tuple
-             or Variable in all cases.
+    Returns
+    -------
+    variable or list/tuple of variables (matches `wrt`)
+        symbolic expression of gradient of `cost` with respect to each
+        of the `wrt` terms.  If an element of `wrt` is not
+        differentiable with respect to the output, then a zero
+        variable is returned.
 
     """
     t0 = time.time()
@@ -421,7 +424,8 @@ def grad(cost, wrt, consider_constant=None,
         from theano import tensor
 
     if cost is None:
-        assert known_grads is not None
+        if known_grads is None:
+            raise AssertionError("cost and known_grads can't both be None.")
 
     if cost is not None and isinstance(cost.type, NullType):
         raise ValueError("Can't differentiate a NaN cost."
@@ -450,7 +454,7 @@ def grad(cost, wrt, consider_constant=None,
     if cost is not None:
         outputs.append(cost)
     if known_grads is not None:
-        outputs.extend(known_grads.keys())
+        outputs.extend(list(known_grads.keys()))
 
     var_to_app_to_idx = _populate_var_to_app_to_idx(
         outputs, wrt, consider_constant)
@@ -467,16 +471,17 @@ def grad(cost, wrt, consider_constant=None,
             g_cost = known_grads[cost]
         else:
             g_cost = _float_ones_like(cost)
-        # g_cost may be Disconnected or NullType. A creative use of the function,
-        # sure, but nonetheless one we can and should support. So before we try
-        # to cast it make sure it even has a dtype
+        # g_cost may be Disconnected or NullType. A creative use of the
+        # function, sure, but nonetheless one we can and should support.
+        # So before we try to cast it make sure it even has a dtype
         if (hasattr(g_cost.type, 'dtype') and
-            cost.type.dtype not in tensor.discrete_dtypes):
-            # Here we enforce the constraint that floating point variables have
-            # the same dtype as their gradient.
-            g_cost = g_cost.astype(cost.type.dtype)
+                cost.type.dtype not in tensor.discrete_dtypes):
+                # Here we enforce the constraint that floating point variables
+                # have the same dtype as their gradient.
+                g_cost = g_cost.astype(cost.type.dtype)
         # DO NOT enforce g_cost to be 0 if cost is an integer.
-        # This is to be enforced by the Op.grad method for the Op that outputs cost.
+        # This is to be enforced by the Op.grad method for the
+        # Op that outputs cost.
         if hasattr(g_cost.type, 'dtype'):
             assert g_cost.type.dtype not in tensor.discrete_dtypes
 
@@ -491,10 +496,10 @@ def grad(cost, wrt, consider_constant=None,
                             ' or sparse theano variable' % str(type(g_var)))
 
         if (not isinstance(g_var.type, (NullType, DisconnectedType)) and
-            'float' not in str(g_var.type.dtype)):
+                'float' not in str(g_var.type.dtype)):
             raise TypeError("Gradients must always be NullType, "
                             "DisconnectedType, or continuous, but grad was "
-                            "given a known_grad of type "+str(g_var.type))
+                            "given a known_grad of type " + str(g_var.type))
 
         # DO NOT check that these gradients are equal to 0 if var is int
         # The gradient is allowed to be non-zero on var in that case
@@ -513,6 +518,17 @@ def grad(cost, wrt, consider_constant=None,
             elif disconnected_inputs == 'warn':
                 warnings.warn(message, stacklevel=2)
             elif disconnected_inputs == 'raise':
+                # Add the var trace
+                tr = getattr(var.tag, 'trace', [])
+                if len(tr) > 0:
+                    message += "\nBacktrace when the node is created:\n"
+
+                    # Print separate message for each element in the list of batcktraces
+                    sio = StringIO()
+                    for subtr in tr:
+                        traceback.print_list(subtr, sio)
+                    message += str(sio.getvalue())
+
                 raise DisconnectedInputError(message)
             else:
                 raise ValueError("Invalid value for keyword "
@@ -545,6 +561,12 @@ def grad(cost, wrt, consider_constant=None,
                                grad_dict, wrt, cost_name)
 
     for i in xrange(len(rval)):
+        if isinstance(rval[i].type, NullType):
+            if null_gradients == 'raise':
+                raise NullTypeGradError("tensor.grad encountered a NaN. " +
+                                        rval[i].type.why_null)
+            else:
+                assert null_gradients == 'return'
         if isinstance(rval[i].type, DisconnectedType):
             handle_disconnected(rval[i])
             if return_disconnected == 'zero':
@@ -698,7 +720,7 @@ def subgraph_grad(wrt, end, start=None, cost=None, details=False):
             for i in range(len(grads)):
                 grads[i] += cost_grads[i]
 
-    pgrads = OrderedDict(zip(params, grads))
+    pgrads = OrderedDict(izip(params, grads))
     # separate wrt from end grads:
     wrt_grads = list(pgrads[k] for k in wrt)
     end_grads = list(pgrads[k] for k in end)
@@ -734,9 +756,9 @@ def _node_to_pattern(node):
             if not isinstance(output_pattern, list):
                 raise TypeError(
                     '%s.connection_pattern should return' %
-                    node.op + ' a list of lists, but element %d' % ii
-                    + 'is %s of type %s.' % (output_pattern,
-                                             type(output_pattern)))
+                    node.op + ' a list of lists, but element %d' % ii +
+                    'is %s of type %s.' % (output_pattern,
+                                           type(output_pattern)))
     else:
         connection_pattern = [[True for output in node.outputs]
                               for ipt in node.inputs]
@@ -839,17 +861,17 @@ def _populate_var_to_app_to_idx(outputs, wrt, consider_constant):
 
             for i, ipt in enumerate(app.inputs):
 
-                #don't process ipt if it is not a true
-                #parent of var
+                # don't process ipt if it is not a true
+                # parent of var
                 if not connection_pattern[i][var_idx]:
                     continue
 
                 if ipt not in var_to_app_to_idx:
                     # This object here *must* be an OrderedDict, because
-                    # we iterate over its keys when adding up the terms of
-                    # the gradient on ipt. If it is a regular dict, the grad
-                    # method will return something that is analytically correct,
-                    # but whose order of doing additions depends on the memory
+                    # we iterate over its keys when adding up the terms of the
+                    # gradient on ipt. If it is a regular dict, the grad method
+                    # will return something that is analytically correct, but
+                    # whose order of doing additions depends on the memory
                     # location of the apply nodes.
                     var_to_app_to_idx[ipt] = OrderedDict()
                 app_to_idx = var_to_app_to_idx[ipt]
@@ -923,8 +945,8 @@ def _populate_grad_dict(var_to_app_to_idx,
         grad_dict: A dictionary mapping variables to their gradients.
                    Should be populated by grad function, which should:
                        -Set the gradient with respect to the cost to 1
-                       -Load all gradients from known_grads, possibly overriding
-                        the cost
+                       -Load all gradients from known_grads, possibly
+                        overriding the cost
                        -Set the gradient for disconnected
                         inputs to a variable with type DisconnectedType()
 
@@ -964,12 +986,12 @@ def _populate_grad_dict(var_to_app_to_idx,
                 input_to_outputs in connection_pattern
             ]
 
-            #List of bools indicating if each output is an integer dtype
+            # List of bools indicating if each output is an integer dtype
             output_is_int = [hasattr(output.type, 'dtype') and
                              output.type.dtype in theano.tensor.discrete_dtypes
                              for output in node.outputs]
 
-            #List of bools indicating if each output is NullType
+            # List of bools indicating if each output is NullType
             ograd_is_nan = [isinstance(output.type, NullType)
                             for output in output_grads]
 
@@ -1004,13 +1026,13 @@ def _populate_grad_dict(var_to_app_to_idx,
                 # call the op's grad method
 
                 # Each Op's grad function requires inputs and output_grads
-                # If the Op destroys any input, but the grad expression uses it,
-                # then chances are the resulting graph will have a dependency
-                # cycle. We avoid this cycle by passing (symbolic) copies of
-                # each destroyed input.
+                # If the Op destroys any input, but the grad expression uses
+                # it, then chances are the resulting graph will have a
+                # dependency cycle. We avoid this cycle by passing (symbolic)
+                # copies of each destroyed input.
                 try:
                     dinputs = [node.inputs[x[0]] for x in
-                               node.op.destroy_map.values()]
+                               itervalues(node.op.destroy_map)]
                 except AttributeError:
                     dinputs = []
 
@@ -1030,15 +1052,16 @@ def _populate_grad_dict(var_to_app_to_idx,
                 # If an output is of an integer dtype, then we just leave it
                 # alone.
                 # DO NOT force integer variables to have zero grad. This causes
-                # bugs where we fail to detect disconnected or undefined gradients.
-                # DO NOT force integer variables to have integer dtype. This is
-                # a violation of the op contract.
+                # bugs where we fail to detect disconnected or undefined
+                # gradients.
+                # DO NOT force integer variables to have integer dtype.
+                # This is a violation of the op contract.
                 new_output_grads = []
                 for o, og in zip(node.outputs, output_grads):
                     o_dt = getattr(o.type, 'dtype', None)
                     og_dt = getattr(og.type, 'dtype', None)
                     if (o_dt not in theano.tensor.discrete_dtypes and
-                        og_dt and o_dt != og_dt):
+                            og_dt and o_dt != og_dt):
                         new_output_grads.append(og.astype(o_dt))
                     else:
                         new_output_grads.append(og)
@@ -1049,7 +1072,7 @@ def _populate_grad_dict(var_to_app_to_idx,
                     o_dt = getattr(o.type, 'dtype', None)
                     ng_dt = getattr(ng.type, 'dtype', None)
                     if (ng_dt is not None and
-                        o_dt not in theano.tensor.discrete_dtypes):
+                            o_dt not in theano.tensor.discrete_dtypes):
                         assert ng_dt == o_dt
 
                 # Someone who had obviously not read the Op contract tried
@@ -1063,14 +1086,15 @@ def _populate_grad_dict(var_to_app_to_idx,
                     assert (getattr(ng.type, 'dtype', None)
                             not in theano.tensor.discrete_dtypes)
 
-                # If config.compute_test_value is turned on, check that the gradients
-                # on the outputs of this node have the right shape.
-                # We also check the gradient on the inputs later--both checks are needed,
-                # because some gradients are only ever specified by the user, not computed
-                # by Op.grad, and some gradients are only computed and returned, but never
-                # passed as another node's output grads.
+                # If config.compute_test_value is turned on, check that the
+                # gradients on the outputs of this node have the right shape.
+                # We also check the gradient on the inputs later--both checks
+                # are needed, because some gradients are only ever specified
+                # by the user, not computed by Op.grad, and some gradients are
+                # only computed and returned, but never passed as another
+                # node's output grads.
                 for idx, packed in enumerate(izip(node.outputs,
-                    new_output_grads)):
+                                             new_output_grads)):
                     orig_output, new_output_grad = packed
                     if not hasattr(orig_output, 'shape'):
                         continue
@@ -1098,19 +1122,31 @@ def _populate_grad_dict(var_to_app_to_idx,
 # We can not enforce this, as AdvancedSubtensor1 has an option to
 # return the sparse grad for optimization reason.
 
-                    #                for ig, i in zip(input_grads, inputs):
-#                    if (not isinstance(ig.type, (DisconnectedType, NullType)) and
-#                        type(ig.type) != type(i.type)):
-#                        raise ValueError(
-#                            "%s returned the wrong type for gradient terms."
-#                            " Sparse inputs must have sparse grads and dense"
-#                            " inputs must have dense grad. Got %s, expected %s" % (
-#                                str(node.op), ig.type, i.type))
+                    #            for ig, i in zip(input_grads, inputs):
+#                if (not isinstance(ig.type, (DisconnectedType, NullType)) and
+#                    type(ig.type) != type(i.type)):
+#                    raise ValueError(
+#                        "%s returned the wrong type for gradient terms."
+#                        " Sparse inputs must have sparse grads and dense"
+#                        " inputs must have dense grad. Got %s, expected %s" %(
+#                            str(node.op), ig.type, i.type))
 
             # must convert to list in case the op returns a tuple
             # we won't be able to post-process out the Nones if it does that
             input_grads = list(input_grads)
 
+            # Need to propagate the NullType gradients; if an input grad is
+            # not disconnected and the corresponding input is connected
+            # to at least one output whose gradient is NullType then the input
+            # grad should be NullType.
+            for inp_idx in range(len(input_grads)):
+                for out_idx in range(len(ograd_is_nan)):
+                    if (ograd_is_nan[out_idx] and
+                            connection_pattern[inp_idx][out_idx] and
+                            not isinstance(input_grads[inp_idx].type,
+                                           DisconnectedType)):
+                        input_grads[inp_idx] = output_grads[out_idx]
+
             # Do type checking on the result
 
             # List of bools indicating if each input only has integer outputs
@@ -1138,7 +1174,8 @@ def _populate_grad_dict(var_to_app_to_idx,
                          'the grad_undefined or grad_unimplemented helper '
                          'functions.') % node.op)
 
-                # Check that the gradient term for this input has the right shape
+                # Check that the gradient term for this input
+                # has the right shape
                 if hasattr(term, 'shape'):
                     orig_ipt = inputs[i]
                     for orig_ipt_v, term_v in get_debug_values(orig_ipt, term):
@@ -1194,8 +1231,8 @@ def _populate_grad_dict(var_to_app_to_idx,
 
                             raise ValueError(msg)
 
-            #Check that op.connection_pattern matches the connectivity
-            #logic driving the op.grad method
+            # Check that op.connection_pattern matches the connectivity
+            # logic driving the op.grad method
             for i, packed in enumerate(zip(inputs, input_grads,
                                            inputs_connected)):
                 ipt, ig, connected = packed
@@ -1223,7 +1260,7 @@ def _populate_grad_dict(var_to_app_to_idx,
                         msg += 'connection_pattern method for it.'
                         warnings.warn(msg)
 
-            #cache the result
+            # cache the result
             term_dict[node] = input_grads
 
         return term_dict[node]
@@ -1233,6 +1270,7 @@ def _populate_grad_dict(var_to_app_to_idx,
         if var not in grad_dict:
             # If var is not in grad_dict already, we must compute it
             if var in var_to_app_to_idx:
+                null_terms = []
                 terms = []
                 node_to_idx = var_to_app_to_idx[var]
                 for node in node_to_idx:
@@ -1247,11 +1285,10 @@ def _populate_grad_dict(var_to_app_to_idx,
                                                          type(term)))
 
                         if isinstance(term.type, NullType):
-                            raise NullTypeGradError("tensor.grad "
-                                                    "encountered a NaN. " +
-                                                    term.type.why_null)
+                            null_terms.append(term)
+                            continue
 
-                        #Don't try to sum up DisconnectedType placeholders
+                        # Don't try to sum up DisconnectedType placeholders
                         if isinstance(term.type, DisconnectedType):
                             continue
 
@@ -1264,7 +1301,11 @@ def _populate_grad_dict(var_to_app_to_idx,
                         terms.append(term)
 
                 # Add up the terms to get the total gradient on this variable
-                if len(terms) > 0:
+                if len(null_terms) > 0:
+                    # At least one term is a NullType : the total gradient
+                    # will also be a NullType
+                    grad_dict[var] = null_terms[0]
+                elif len(terms) > 0:
                     # the next line is like sum(terms) but doesn't add an
                     # extraneous TensorConstant(0)
                     grad_dict[var] = reduce(lambda x, y: x + y, terms)
@@ -1382,20 +1423,21 @@ class numeric_grad(object):
         # if not dtypes == [dtypes[0]] * len(apt):
         #      raise TypeError('All function arguments must have same dtype')
 
-        total_size = __builtin__.sum(prod(sh) for sh in shapes)
+        total_size = builtins.sum(prod(sh) for sh in shapes)
 
-        working_dtype = __builtin__.min((self.type_eps[dt], dt)
-                                        for dt in dtypes)[1]
+        working_dtype = builtins.min(
+            (self.type_eps[dt], dt) for dt in dtypes)[1]
 
         # create un-initialized memory
         x = numpy.ndarray((total_size,), dtype=working_dtype)
-        if (not out_type is None) and (out_type.startswith('complex')):
+        # (not out_type is None) --> (out_type is not None) ???
+        if (out_type is not None) and (out_type.startswith('complex')):
             gx = numpy.ndarray((total_size,), dtype=out_type)
         else:
             gx = numpy.ndarray((total_size,), dtype=working_dtype)
 
         if eps is None:
-            eps = __builtin__.max(self.type_eps[dt] for dt in dtypes)
+            eps = builtins.max(self.type_eps[dt] for dt in dtypes)
 
         # set up aliases so that apt[i] is backed by memory in x
         # and self.gf is backed by memory in gx
@@ -1519,8 +1561,8 @@ def verify_grad(fun, pt, n_tests=2, rng=None, eps=None,
 
     Example:
         >>> verify_grad(theano.tensor.tanh,
-                        (numpy.asarray([[2,3,4], [-1, 3.3, 9.9]]),),
-                        rng=numpy.random)
+        ...             (numpy.asarray([[2,3,4], [-1, 3.3, 9.9]]),),
+        ...             rng=numpy.random)
 
     Raises an Exception if the difference between the analytic gradient and
     numerical gradient (computed through the Finite Difference Method) of a
@@ -1580,9 +1622,9 @@ def verify_grad(fun, pt, n_tests=2, rng=None, eps=None,
         float64=1e-4)
 
     if abs_tol is None:
-        abs_tol = __builtin__.max(_type_tol[str(p.dtype)] for p in pt)
+        abs_tol = builtins.max(_type_tol[str(p.dtype)] for p in pt)
     if rel_tol is None:
-        rel_tol = __builtin__.max(_type_tol[str(p.dtype)] for p in pt)
+        rel_tol = builtins.max(_type_tol[str(p.dtype)] for p in pt)
 
     if rng is None:
         raise TypeError(('rng should be a valid instance of '
@@ -1593,15 +1635,10 @@ def verify_grad(fun, pt, n_tests=2, rng=None, eps=None,
 
     # We allow input downcast in function, because numeric_grad works in the
     # most precise dtype used among the inputs, so we may need to cast some.
-    def function(inputs, output):
-        if mode is None:
-            f = compile.function(inputs, output, accept_inplace=True,
-                                 allow_input_downcast=True,
-                                 on_unused_input='ignore')
-        else:
-            f = compile.function(inputs, output, accept_inplace=True,
-                                 allow_input_downcast=True, mode=mode,
-                                 on_unused_input='ignore')
+    def function(inputs, output, name):
+        f = compile.function(inputs, output, accept_inplace=True,
+                             allow_input_downcast=True, mode=mode,
+                             on_unused_input='ignore', name=name)
         return f
 
     tensor_pt = [
@@ -1620,7 +1657,7 @@ def verify_grad(fun, pt, n_tests=2, rng=None, eps=None,
         # but this doesn't handle the case where not all the outputs are
         # differentiable... so I leave this as TODO for now -JB.
 
-    o_fn = function(tensor_pt, o_output)
+    o_fn = function(tensor_pt, o_output, name='gradient.py fwd')
     o_fn_out = o_fn(*[p.copy() for p in pt])
 
     if isinstance(o_fn_out, tuple) or isinstance(o_fn_out, list):
@@ -1644,12 +1681,13 @@ def verify_grad(fun, pt, n_tests=2, rng=None, eps=None,
     # This sum() is defined above, it's not the builtin sum.
     cost = theano.tensor.sum(t_r * o_output)
 
-    cost_fn = function(tensor_pt, cost)
+    cost_fn = function(tensor_pt, cost, name='gradient.py cost')
 
     symbolic_grad = grad(cost, tensor_pt,
                          disconnected_inputs='ignore')
 
-    grad_fn = function(tensor_pt, symbolic_grad)
+    grad_fn = function(tensor_pt, symbolic_grad,
+                       name='gradient.py symbolic grad')
 
     for test_num in xrange(n_tests):
         try:
@@ -1673,7 +1711,7 @@ def verify_grad(fun, pt, n_tests=2, rng=None, eps=None,
             # get new random projection for next test
             if test_num < n_tests - 1:
                 t_r.set_value(random_projection(), borrow=True)
-        except Exception, e:
+        except Exception as e:
             e.args += ("\nThe error happened with the following inputs:", pt,
                        "\nThe value of eps is:", eps,
                        "\nThe out_type is:", out_type)
@@ -1734,10 +1772,10 @@ def jacobian(expression, wrt, consider_constant=None,
     from theano.tensor import arange
     # Check inputs have the right format
     assert isinstance(expression, Variable), \
-            "tensor.jacobian expects a Variable as `expression`"
+        "tensor.jacobian expects a Variable as `expression`"
     assert expression.ndim < 2, \
-            ("tensor.jacobian expects a 1 dimensional variable as "
-             "`expression`. If not use flatten to make it a vector")
+        ("tensor.jacobian expects a 1 dimensional variable as "
+         "`expression`. If not use flatten to make it a vector")
 
     using_list = isinstance(wrt, list)
     using_tuple = isinstance(wrt, tuple)
@@ -1774,9 +1812,9 @@ def jacobian(expression, wrt, consider_constant=None,
                                   sequences=arange(expression.shape[0]),
                                   non_sequences=[expression] + wrt)
     assert not updates, \
-            ("Scan has returned a list of updates. This should not "
-             "happen! Report this to theano-users (also include the "
-             "script that generated the error)")
+        ("Scan has returned a list of updates. This should not "
+         "happen! Report this to theano-users (also include the "
+         "script that generated the error)")
     return format_as(using_list, using_tuple, jacobs)
 
 
@@ -1808,9 +1846,9 @@ def hessian(cost, wrt, consider_constant=None,
     from theano.tensor import arange
     # Check inputs have the right format
     assert isinstance(cost, Variable), \
-            "tensor.hessian expects a Variable as `cost`"
+        "tensor.hessian expects a Variable as `cost`"
     assert cost.ndim == 0, \
-            "tensor.hessian expects a 0 dimensional variable as `cost`"
+        "tensor.hessian expects a 0 dimensional variable as `cost`"
 
     using_list = isinstance(wrt, list)
     using_tuple = isinstance(wrt, tuple)
@@ -1823,10 +1861,10 @@ def hessian(cost, wrt, consider_constant=None,
     hessians = []
     for input in wrt:
         assert isinstance(input, Variable), \
-                "tensor.hessian expects a (list of) Variable as `wrt`"
+            "tensor.hessian expects a (list of) Variable as `wrt`"
         assert input.ndim == 1, \
-                "tensor.hessian expects a (list of) 1 dimensional variable "\
-                "as `wrt`"
+            "tensor.hessian expects a (list of) 1 dimensional variable "\
+            "as `wrt`"
         expr = grad(cost, input, consider_constant=consider_constant,
                     disconnected_inputs=disconnected_inputs)
 
@@ -1834,16 +1872,16 @@ def hessian(cost, wrt, consider_constant=None,
         # even if they are connected to cost.
         # This should not be an error.
         hess, updates = theano.scan(lambda i, y, x: grad(
-                            y[i],
-                            x,
-                            consider_constant=consider_constant,
-                            disconnected_inputs='ignore'),
-                       sequences=arange(expr.shape[0]),
-                       non_sequences=[expr, input])
+            y[i],
+            x,
+            consider_constant=consider_constant,
+            disconnected_inputs='ignore'),
+            sequences=arange(expr.shape[0]),
+            non_sequences=[expr, input])
         assert not updates, \
-                ("Scan has returned a list of updates. This should not "
-                 "happen! Report this to theano-users (also include the "
-                 "script that generated the error)")
+            ("Scan has returned a list of updates. This should not "
+             "happen! Report this to theano-users (also include the "
+             "script that generated the error)")
         hessians.append(hess)
     return format_as(using_list, using_tuple, hessians)
 
@@ -1888,7 +1926,6 @@ consider_constant_ = ConsiderConstant()
 
 # I create a function only to have the doc show well.
 def consider_constant(x):
-
     """
     DEPRECATED: use zero_grad() or disconnected_grad() instead.
 
@@ -1923,7 +1960,6 @@ zero_grad_ = ZeroGrad()
 
 
 def zero_grad(x):
-
     """
     Consider an expression constant when computing gradients.
 
@@ -1952,7 +1988,6 @@ disconnected_grad_ = DisconnectedGrad()
 
 
 def disconnected_grad(x):
-
     """
     Consider an expression constant when computing gradients,
     while effectively not backpropagating through it.
@@ -1977,6 +2012,7 @@ def disconnected_grad(x):
 class GradClip(ViewOp):
     # See doc in user fct grad_clip
     __props__ = ()
+
     def __init__(self, clip_lower_bound, clip_upper_bound):
         # We do not put those member in __eq__ or __hash__
         # as they do not influence the perform of this op.
@@ -1989,6 +2025,7 @@ class GradClip(ViewOp):
                                    self.clip_upper_bound)
                 for g_out in g_outs]
 
+
 def grad_clip(x, lower_bound, upper_bound):
     """
     This op do a view in the forward, but clip the gradient.
@@ -1998,7 +2035,7 @@ def grad_clip(x, lower_bound, upper_bound):
     :param x: the variable we want its gradient inputs clipped
     :param lower_bound: The lower bound of the gradient value
     :param upper_bound: The upper bound of the gradient value.
-    
+
     :examples:
 
         x = theano.tensor.scalar()
@@ -2008,7 +2045,7 @@ def grad_clip(x, lower_bound, upper_bound):
 
         f = theano.function([x], outputs = [z, z2])
 
-        print f(2.0)  # output (1.0, 4.0)
+        print(f(2.0))  # output (1.0, 4.0)
 
     :note: We register an opt in tensor/opt.py that remove the GradClip.
        So it have 0 cost in the forward and only do work in the grad.
