@@ -1,3 +1,4 @@
+from __future__ import print_function
 import atexit
 import errno
 import logging
@@ -5,6 +6,7 @@ import os
 import shutil
 import stat
 import sys
+import warnings
 
 import theano
 from theano.compat import get_unbound_function
@@ -12,8 +14,10 @@ from theano.compile import optdb
 from theano.gof import EquilibriumDB, SequenceDB
 from theano.gof.cmodule import get_lib_extension
 from theano.gof.compilelock import get_lock, release_lock
-from theano.configparser import config, AddConfigVar, StrParam, BoolParam
-import nvcc_compiler
+from theano import config
+from . import nvcc_compiler
+
+from theano.tensor.basic import register_transfer
 
 # ignore_newtrees is to speed the optimization as this is the pattern
 # we use for optimization. Otherwise, we can iterate 100s of time on
@@ -26,35 +30,20 @@ def register_opt(*tags, **kwargs):
     if any([not isinstance(t, str) for t in tags]):
         raise RuntimeError("Bad call to register_opt."
                            " All tags must be strings.", tags)
+
     def f(local_opt):
         name = (kwargs and kwargs.pop('name')) or local_opt.__name__
         gpu_optimizer.register(name, local_opt, 'fast_run', 'fast_compile',
-                               'gpu', *tags)
+                               'gpu', *tags, **kwargs)
         return local_opt
     return f
 
 
-
 _logger_name = 'theano.sandbox.cuda'
 _logger = logging.getLogger(_logger_name)
 
-AddConfigVar('pycuda.init',
-        """If True, always initialize PyCUDA when Theano want to
-           initilize the GPU.  Currently, we must always initialize
-           PyCUDA before Theano do it.  Setting this flag to True,
-           ensure that, but always import PyCUDA.  It can be done
-           manually by importing theano.misc.pycuda_init before theano
-           initialize the GPU device.
-             """,
-        BoolParam(False),
-        in_c_key=False)
-
-AddConfigVar('cublas.lib',
-        """Name of the cuda blas library for the linker.""",
-        StrParam('cublas'))
-
-#is_nvcc_available called here to initialize global vars in
-#nvcc_compiler module
+# is_nvcc_available called here to initialize global vars in
+# nvcc_compiler module
 nvcc_compiler.is_nvcc_available()
 
 # Compile cuda_ndarray.cu
@@ -70,24 +59,26 @@ cuda_available = True
 # Global variable to avoid displaying the same warning multiple times.
 cuda_warning_is_displayed = False
 
-#This variable is set to True when we enable cuda.(i.e. when use() is called)
+# This variable is set to True when we enable cuda.(i.e. when use() is called)
 cuda_enabled = False
 
 
 # Code factorized within a function so that it may be called from multiple
 # places (which is not currently the case, but may be useful in the future).
 def set_cuda_disabled():
-    """Function used to disable cuda.
+    """
+    Function used to disable cuda.
 
     A warning is displayed, so that the user is aware that cuda-based code is
     not going to work.
     Note that there is no point calling this function from outside of
     `cuda.__init__`, since it has no effect once the module is loaded.
+
     """
     global cuda_available, cuda_warning_is_displayed
     cuda_available = False
 
-#cuda_ndarray compile and import
+# cuda_ndarray compile and import
 cuda_path = os.path.abspath(os.path.split(__file__)[0])
 
 cuda_ndarray_loc = os.path.join(config.compiledir, 'cuda_ndarray')
@@ -99,13 +90,16 @@ libcuda_ndarray_so = os.path.join(cuda_ndarray_loc,
 
 def try_import():
     """
-    load the cuda_ndarray module if present and up to date
-    return True if loaded correctly, otherwise return False
+    Load the cuda_ndarray module if present and up to date.
+    Return True if loaded correctly, otherwise return False.
+
     """
     cuda_files = (
         'cuda_ndarray.cu',
         'cuda_ndarray.cuh',
         'conv_full_kernel.cu',
+        'cnmem.h',
+        'cnmem.cpp',
         'conv_kernel.cu')
     stat_times = [os.stat(os.path.join(cuda_path, cuda_file))[stat.ST_MTIME]
                   for cuda_file in cuda_files]
@@ -169,17 +163,18 @@ if compile_cuda_ndarray and cuda_available:
                         tmpdir = os.environ['TMPDIR']
                         if not os.path.exists(tmpdir):
                             os.makedirs(tmpdir)
-
                     compiler = nvcc_compiler.NVCC_compiler()
+                    preargs = ['-O3'] + compiler.compile_args()
                     compiler.compile_str(
                             'cuda_ndarray',
                             code,
                             location=cuda_ndarray_loc,
                             include_dirs=[cuda_path],
                             libs=[config.cublas.lib],
-                            preargs=['-O3'] + compiler.compile_args())
+                            preargs=preargs,
+                    )
                     from cuda_ndarray.cuda_ndarray import *
-            except Exception, e:
+            except Exception as e:
                 _logger.error("Failed to compile cuda_ndarray.cu: %s", str(e))
                 set_cuda_disabled()
     finally:
@@ -199,6 +194,7 @@ if cuda_available:
     def ok():
         """
         Check if an existing library exists and can be read.
+
         """
         try:
             open(libcuda_ndarray_so).close()
@@ -212,7 +208,7 @@ if cuda_available:
         else:
             try:
                 os.symlink(cuda_ndarray_so, libcuda_ndarray_so)
-            except OSError, e:
+            except OSError as e:
                 # This may happen for instance when running multiple
                 # concurrent jobs, if two of them try to create the
                 # symlink simultaneously.
@@ -229,11 +225,11 @@ if cuda_available:
         cuda_initialization_error_message = ""
 # actively closing our gpu session presents segfault-on-exit on some systems
         atexit.register(gpu_shutdown)
-    except EnvironmentError, e:
+    except EnvironmentError as e:
         cuda_available = False
         cuda_initialization_error_message = " ".join(e.args)
 else:
-    cuda_initialization_error_message = 'cuda unavilable'
+    cuda_initialization_error_message = 'cuda unavailable'
 
 
 class GpuOp(theano.gof.Op):
@@ -246,15 +242,16 @@ class GpuOp(theano.gof.Op):
 
     It is defined in __init__.py so that it exists even when `cuda_available`
     is False (this is necessary to avoid breaking the test suite).
+
     """
 
     def make_thunk(self, node, storage_map, compute_map, no_recycling):
-        if theano.sandbox.cuda.use.device_number is None:
-            theano.sandbox.cuda.use("gpu",
-                                    force=True,
-                                    default_to_move_computation_to_gpu=False,
-                                    move_shared_float32_to_gpu=False,
-                                    enable_cuda=False)
+        if use.device_number is None:
+            use("gpu",
+                force=True,
+                default_to_move_computation_to_gpu=False,
+                move_shared_float32_to_gpu=False,
+                enable_cuda=False)
         return super(GpuOp, self).make_thunk(node, storage_map,
                                              compute_map, no_recycling)
 
@@ -270,6 +267,154 @@ from theano.sandbox.cuda.var import (CudaNdarrayVariable,
 from theano.sandbox.cuda.type import CudaNdarrayType
 
 
+def dnn_available():
+    if config.dnn.enabled == "False":
+        dnn_available.avail = False
+        dnn_available.msg = "Disabled by dnn.enabled flag"
+    if dnn_available.avail is None and not cuda_available:
+        dnn_available.msg = "CUDA not available"
+        dnn_available.avail = False
+    elif dnn_available.avail is None:
+        dev = active_device_number()
+        if device_properties(dev)['major'] < 3:
+            dnn_available.msg = "Device not supported"
+            dnn_available.avail = False
+        else:
+            preambule = """
+#include <stdio.h>
+#include <cuda.h>
+#include <cudnn.h>
+#include <cudnn_helper.h>
+            """
+
+            body = """
+cudnnHandle_t _handle = NULL;
+cudnnStatus_t err;
+if ((err = cudnnCreate(&_handle)) != CUDNN_STATUS_SUCCESS) {
+  fprintf(stderr, "could not create cuDNN handle: %s",
+          cudnnGetErrorString(err));
+  return 1;
+}
+"""
+            params = ["-l", "cudnn", "-I" + os.path.dirname(__file__)]
+            if config.dnn.include_path:
+                params.append("-I" + config.dnn.include_path)
+            if config.dnn.library_path:
+                params.append("-L" + config.dnn.library_path)
+            if config.nvcc.compiler_bindir:
+                params.extend(['--compiler-bindir',
+                               config.nvcc.compiler_bindir])
+            # Do not run here the test program. It would run on the
+            # default gpu, not the one selected by the user. If mixed
+            # GPU are installed or if the GPUs are configured in
+            # exclusive mode, this cause bad detection.
+            comp, out, err = nvcc_compiler.NVCC_compiler.try_flags(
+                flag_list=params, preambule=preambule, body=body,
+                try_run=False, output=True)
+
+            dnn_available.avail = comp
+            if not dnn_available.avail:
+                dnn_available.msg = (
+                    "Can not compile with cuDNN. We got this error:\n" +
+                    str(err))
+            else:
+                # If we can compile, check that we can import and run.
+                v = dnn_version()
+                if isinstance(v, tuple) and v[0] != v[1]:
+                    dnn_available.avail = False
+                    dnn_available.msg = ("Mixed dnn version. The header is"
+                                         " from one version, but we link with"
+                                         " a different version %s" % str(v))
+                    raise RuntimeError(dnn_available.msg)
+                if v == -1 or v[0] < 4007:
+                    # 4007 is the final release of cudnn v4
+                    dnn_available.avail = False
+                    dnn_available.msg = "Version is too old. Update to v5, was %d." % v[0]
+                    raise RuntimeError(dnn_available.msg)
+                else:
+                    dnn_available.avail = comp
+    if config.dnn.enabled == "True":
+        if not dnn_available.avail:
+            raise RuntimeError(
+                "You enabled cuDNN, but we aren't able to use it: %s" %
+                dnn_available.msg)
+    return dnn_available.avail
+
+
+dnn_available.avail = None
+dnn_available.msg = None
+
+
+class DnnVersion(GpuOp):
+    def c_compiler(self):
+        return nvcc_compiler.NVCC_compiler
+
+    def c_headers(self):
+        return ['cudnn.h']
+
+    def c_header_dirs(self):
+        return [config.dnn.include_path]
+
+    def c_libraries(self):
+        return ['cudnn']
+
+    def c_lib_dirs(self):
+        return [config.dnn.library_path]
+
+    def c_compile_args(self):
+        return ['-Wl,-rpath,' + config.dnn.library_path]
+
+    def c_support_code(self):
+        return """
+#if PY_MAJOR_VERSION >= 3
+#define PyInt_FromLong PyLong_FromLong
+#endif
+"""
+
+    def make_node(self):
+        return theano.gof.Apply(self, [], [theano.gof.Generic()()])
+
+    def c_code(self, node, name, inputs, outputs, sub):
+        o = outputs[0]
+        return """
+        #if defined(CUDNN_VERSION)
+        %(o)s = PyTuple_Pack(2, PyInt_FromLong(CUDNN_VERSION), PyInt_FromLong(cudnnGetVersion()));
+        #else
+        %(o)s = PyInt_FromLong(-1);
+        #endif
+        """ % locals()
+
+    def do_constant_folding(self, node):
+        # Needed as we do not want to cache this information.
+        return False
+
+    def c_code_cache_version(self):
+        # Not needed, but make it clear that we do not want to cache this.
+        return None
+
+
+def dnn_version():
+    """Return the current cuDNN version we compile with.
+
+    This returns a tuple with the header version and the library
+    version we link with. For older cudnn version without version
+    information, we return -1.
+
+    """
+    if not dnn_available():
+        raise Exception(
+            "We can't determine the cudnn version as it is not available",
+            dnn_available.msg)
+
+    if dnn_version.v is None:
+        f = theano.function([], DnnVersion()(),
+                            theano.Mode(optimizer=None),
+                            profile=False)
+        dnn_version.v = f()
+    return dnn_version.v
+dnn_version.v = None
+
+
 if cuda_available:
     # check if their is an old cuda_ndarray that was loading instead of the one
     # we compiled!
@@ -291,7 +436,7 @@ if cuda_available:
             GpuDimShuffle, GpuCAReduce, GpuReshape, GpuContiguous,
             GpuSubtensor, GpuIncSubtensor,
             GpuAdvancedSubtensor1, GpuAdvancedIncSubtensor1,
-            GpuFlatten, GpuShape, GpuAlloc, GpuSplit,
+            gpu_flatten, GpuFlatten, GpuShape, GpuAlloc, GpuAllocEmpty, GpuSplit,
             GpuJoin, fscalar, fvector, fmatrix, frow, fcol,
             ftensor3, ftensor4,
             scalar, vector, matrix, row, col,
@@ -302,6 +447,12 @@ if cuda_available:
     from . import opt, dnn
     from .rng_curand import CURAND_RandomStreams
 
+    def transfer(x, target):
+        if target == 'gpu':
+            return as_cuda_ndarray_variable(x)
+
+    register_transfer(transfer)
+
 
 def use(device,
         force=False,
@@ -311,18 +462,23 @@ def use(device,
         test_driver=True):
     """
     Error and warning about CUDA should be displayed only when this
-    function is called.  We need to be able to load this module only
+    function is called. We need to be able to load this module only
     to check if it is available!
 
-    :param device: string "cpu", "gpu", "gpuN" (N is the device number to use)
-    :param force: Will always raise an exception if we can't use the gpu.
-    :param default_to_move_computation_to_gpu: If gpu init succeeded, enable by
-                                               default optimizations to move
-                                               computations to the gpu
-    :param move_shared_float32_to_gpu: If gpu init succeeded, put new shared
-                                       variables in float32 on the gpu.
-    :param enable_cuda: If the gpu is correctly enabled,
-                        set the variable cuda_enabled to True.
+    Parameters
+    ----------
+    device : string
+        "cpu", "gpu", "gpuN" (N is the device number to use).
+    force
+        Will always raise an exception if we can't use the gpu.
+    default_to_move_computation_to_gpu
+        If gpu init succeeded, enable by default optimizations to move
+        computations to the gpu.
+    move_shared_float32_to_gpu
+        If gpu init succeeded, put new shared variables in float32 on the gpu.
+    enable_cuda
+        If the gpu is correctly enabled, set the variable cuda_enabled to True.
+
     """
     global cuda_enabled, cuda_initialization_error_message
     if force and not cuda_available and device.startswith('gpu'):
@@ -374,11 +530,16 @@ def use(device,
             pycuda_init_dev = theano.misc.pycuda_init.pycuda_available
 
         try:
-            if (device != 'gpu') and not pycuda_init_dev:
+            if pycuda_init_dev:
+                use.device_number = active_device_number()
+                # This is needed to initialize the cublas handle.
+                gpu_init(use.device_number, config.lib.cnmem)
+            elif(device != 'gpu'):
                 assert isinstance(device, int)
-                gpu_init(device)
+                gpu_init(device, config.lib.cnmem)
                 use.device_number = device
-                assert active_device_number() == device
+                active_device = active_device_number()
+                assert active_device == device, (active_device, device)
             else:
                 # This mean the driver should select the GPU.  As we
                 # need to get the device number now, we force the
@@ -386,10 +547,14 @@ def use(device,
                 # query the active GPU. If we check the active GPU before
                 # the device is initialized we will always receive 0
                 # event if another device is selected later.
-                cuda_ndarray.cuda_ndarray.CudaNdarray.zeros((2, 3))
+                if not hasattr(cuda_ndarray.cuda_ndarray, 'select_a_gpu'):
+                    raise Exception(
+                        "Delete your Theano cache. The automatic"
+                        " recompilation did not work.")
+                cuda_ndarray.cuda_ndarray.select_a_gpu()
                 use.device_number = active_device_number()
                 # This is needed to initialize the cublas handle.
-                gpu_init(use.device_number)
+                gpu_init(use.device_number, config.lib.cnmem)
 
             if test_driver:
                 import theano.sandbox.cuda.tests.test_driver
@@ -402,8 +567,38 @@ def use(device,
                                  " this property")
 
             if config.print_active_device:
-                print >> sys.stderr, "Using gpu device %d: %s" % (
-                        active_device_number(), active_device_name())
+                if config.lib.cnmem:
+                    if config.lib.cnmem > 1:
+                        cnmem_enabled = "enabled with initial size: %d MB" % config.lib.cnmem
+                    else:
+                        cnmem = min(config.lib.cnmem, 0.95) * 100
+                        cnmem_enabled = "enabled with initial size: %.1f%% of memory" % cnmem
+                else:
+                    cnmem_enabled = "disabled"
+                cudnn_version = "not available"
+                warn = None
+                try:
+                    if dnn_available():
+                        (hdr_v, runtime_v) = dnn_version()
+                        cudnn_version = runtime_v
+                        # 5100 should not print warning with cudnn 5 final.
+                        if cudnn_version > 5100:
+                            warn = ("Your cuDNN version is more recent than the one"
+                                    " Theano officially supports."
+                                    " If you see any problems, try updating Theano or"
+                                    " downgrading cuDNN to version 5.")
+                except Exception:
+                    cudnn_version = dnn_available.msg
+                print("Using gpu device %d: %s (CNMeM is %s, cuDNN %s)" % (
+                    active_device_number(),
+                    active_device_name(),
+                    cnmem_enabled,
+                    cudnn_version,),
+                      file=sys.stderr)
+                if warn:
+                    import warnings
+                    warnings.warn(warn)
+
             if device_properties(use.device_number)['regsPerBlock'] < 16384:
                 # We will try to use too much register per bloc at many places
                 # when there is only 8k register per multi-processor.
@@ -414,7 +609,7 @@ def use(device,
                         " crash when we try to use features"
                         " that your GPU does not support.")
 
-        except (EnvironmentError, ValueError, RuntimeError), e:
+        except (EnvironmentError, ValueError, RuntimeError) as e:
             _logger.error(("ERROR: Not using GPU."
                            " Initialisation of device %s failed:\n%s"),
                           str(device), e)
@@ -436,20 +631,22 @@ def use(device,
         cuda_enabled = True
 
     if default_to_move_computation_to_gpu:
+        # Do not add inplace tag here. We do not want to
+        # enable/disable gpu opt based on the inplace tag.
         optdb.add_tags('gpu_opt',
                        'fast_compile',
-                       'fast_run',
-                       'inplace')
+                       'fast_run')
         optdb.add_tags('gpu_after_fusion',
-                       'fast_run',
-                       'inplace')
+                       'fast_run')
+        optdb.add_tags('gpu_scanOp_make_inplace',
+                       'fast_run')
 
     if force:
         try:
-            #in case the device if just gpu,
+            # in case the device if just gpu,
             # we check that the driver init it correctly.
             cuda_ndarray.cuda_ndarray.CudaNdarray.zeros((5, 5))
-        except (Exception, NameError), e:
+        except (Exception, NameError) as e:
             # NameError when no gpu present as cuda_ndarray is not loaded.
             e.args += ("ERROR: GPU forced but failed. ",)
             raise
@@ -458,7 +655,7 @@ use.device_number = None
 
 def unuse():
     """
-    This undo what was done by the call to
+    This undo what was done by the call to.
 
     use('gpu[0-9]', default_to_move_computation_to_gpu=True,
         move_shared_float32_to_gpu=True,
@@ -466,23 +663,26 @@ def unuse():
 
     This is used in Pylearn2 tests to enable/disable the GPU when needed.
 
-    After this call, the rest of Theano think the GPU shouldn't be used by default.
+    After this call, the rest of Theano think the GPU shouldn't be used by
+    default.
+
     """
     global cuda_enabled
     cuda_enabled = False
     handle_shared_float32(False)
     optdb.remove_tags('gpu_opt',
-                   'fast_run',
-                   'inplace')
+                      'fast_compile',
+                      'fast_run')
     optdb.remove_tags('gpu_after_fusion',
-                   'fast_run',
-                   'inplace')
+                      'fast_run')
 
 
 def handle_shared_float32(tf):
-    """Set the default shared type for float32 tensor to CudaNdarrayType
+    """
+    Set the default shared type for float32 tensor to CudaNdarrayType.
 
     This function is intended to be called from use(gpu_index), not directly.
+
     """
     if tf:
         theano.compile.shared_constructor(float32_shared_constructor)
@@ -495,7 +695,7 @@ def handle_shared_float32(tf):
 # import dependency. So we also test it in the file theano/__init__.py
 if config.device.startswith('gpu'):
     use(device=config.device, force=config.force_device, test_driver=False)
-elif config.init_gpu_device:
+elif config.init_gpu_device.startswith('gpu'):
     assert config.device == "cpu", (
         "We can use the Theano flag init_gpu_device"
         " only when the Theano flag device=='cpu'")
