@@ -16,15 +16,14 @@ if cuda_ndarray.cuda_available == False:
 
 import theano.sandbox.cuda as tcn
 
-from theano.tensor.signal.downsample import (DownsampleFactorMax,
-        DownsampleFactorMaxGrad, DownsampleFactorMaxGradGrad)
-from theano.compat.python2x import any
+from theano.tensor.signal.pool import (Pool,
+        PoolGrad, DownsampleFactorMaxGradGrad)
 
 import theano.compile.mode
 from theano.tensor.tests.test_blas import BaseGemv, TestBlasStrides, TestGer
 from theano.sandbox.cuda.blas import gpu_gemv_no_inplace, gpu_gemv_inplace
 from theano.sandbox.cuda.blas import gpu_ger_inplace, gpu_ger_no_inplace
-
+from theano.sandbox.cuda.blas import batched_dot, GpuBatchedDot
 
 if theano.config.mode == 'FAST_COMPILE':
     mode_with_gpu = theano.compile.mode.get_mode('FAST_RUN').including('gpu')
@@ -34,7 +33,7 @@ else:
     mode_with_gpu = theano.compile.mode.get_default_mode().including('gpu')
     mode_without_gpu = theano.compile.mode.get_default_mode().excluding('gpu')
 
-#The CPU tests already compare C/Py, so we only check C/GPU
+# The CPU tests already compare C/Py, so we only check C/GPU
 mode_with_gpu = copy.copy(mode_with_gpu)
 mode_without_gpu = copy.copy(mode_without_gpu)
 mode_with_gpu.check_py_code = False
@@ -45,6 +44,93 @@ def my_rand(*shape):
     return theano._asarray(numpy.random.rand(*shape), dtype='float32')
 
 
+class TestBatchedDot(unittest_tools.InferShapeTester):
+    mode = mode_with_gpu
+
+    def test_batched_dot_correctness(self):
+        # test both implementations
+        for threshold in [0, 100]:
+            batched_dot = GpuBatchedDot(stream_threshold=threshold)
+
+            def cmp(a_shp, b_shp):
+
+                a=numpy.random.randn(*a_shp).astype(numpy.float32)
+                b=numpy.random.randn(*b_shp).astype(numpy.float32)
+
+                x=tensor.ftensor3()
+                y=tensor.ftensor3()
+
+                f=theano.function([x,y], batched_dot(x,y), mode=mode_with_gpu)
+
+                z0=numpy.asarray(f(a,b))
+
+                ga = cuda_ndarray.CudaNdarray(a)
+                gb = cuda_ndarray.CudaNdarray(b)
+
+                z1=numpy.asarray(f(ga,gb))
+
+                z_test = numpy.sum(a[:,:,:,None]*b[:,None,:,:],axis=-2)
+
+                unittest_tools.assert_allclose(z0, z_test)
+                unittest_tools.assert_allclose(z1, z_test)
+
+            cmp((5,4,3), (5,3,2))
+            cmp((5,3,3), (5,3,3))
+            cmp((5,2,6), (5,6,3))
+
+            # Test dimensions of 0
+            cmp((0,2,6), (0,6,3))
+            cmp((5,0,3), (5,3,2))
+            cmp((5,4,0), (5,0,2))
+            cmp((5,4,3), (5,3,0))
+            cmp((0,0,0), (0,0,0))
+
+            # Test dimensions of 1
+            cmp((1,2,6), (1,6,3))
+            cmp((5,1,3), (5,3,2))
+            cmp((5,4,1), (5,1,2))
+            cmp((5,4,3), (5,3,1))
+
+    def test_batched_dot_errors(self):
+
+        def fail(a_shp, b_shp):
+
+            a=numpy.random.randn(*a_shp).astype(numpy.float32)
+            b=numpy.random.randn(*b_shp).astype(numpy.float32)
+
+            x=tensor.ftensor3()
+            y=tensor.ftensor3()
+
+            f=theano.function([x,y], batched_dot(x,y), mode=mode_with_gpu)
+
+            z = f(a,b)
+
+        # Different batch size
+        self.assertRaises(RuntimeError, fail, (5,4,3), (6,3,2))
+
+        # Shape mismatch
+        self.assertRaises(RuntimeError, fail, (5,4,3), (5,2,2))
+
+    def test_batched_dot_gradient(self):
+        for threshold in [0, 100]:
+            unittest_tools.verify_grad(
+                GpuBatchedDot(stream_threshold=threshold),
+                [numpy.random.randn(5,7,2).astype(numpy.float32),
+                 numpy.random.randn(5,2,6).astype(numpy.float32)],
+                mode=mode_with_gpu)
+
+    def test_infer_shape(self):
+        # only matrix/matrix is supported
+        admat = tensor.ftensor3()
+        bdmat = tensor.ftensor3()
+        admat_val = my_rand(7, 4, 5)
+        bdmat_val = my_rand(7, 5, 3)
+        self._compile_and_check([admat, bdmat],
+                                [GpuBatchedDot()(admat, bdmat)],
+                                [admat_val, bdmat_val],
+                                GpuBatchedDot)
+
+
 def test_dot22():
     def cmp(a_shp, b_shp):
         a0 = my_rand(*a_shp)
@@ -91,15 +177,23 @@ def test_dot22scalar():
                 [a, b],
                 tensor.dot(a, b) * numpy.asarray(4, 'float32'))
         t = f.maker.fgraph.toposort()
-        assert len(t) == 4
-        assert isinstance(t[0].op, tcn.GpuFromHost)
-        assert isinstance(t[1].op, tcn.GpuFromHost)
-        assert isinstance(t[2].op, tcn.blas.GpuDot22Scalar)
-        assert isinstance(t[3].op, tcn.HostFromGpu)
+        assert any([isinstance(n.op, tcn.blas.GpuDot22Scalar) for n in t])
+#        assert any([isinstance(n.op, tcn.basic_ops.GpuAllocEmpty)
+#                    for n in t])
         assert numpy.allclose(f(av, bv), f2(av, bv))
 
         f = theano.function([a, b, scalar], tensor.dot(a, b) * scalar,
-                mode=mode_with_gpu)
+                            mode=mode_with_gpu)
+        f2 = theano.function([a, b, scalar], tensor.dot(a, b) * scalar)
+        t = f.maker.fgraph.toposort()
+        assert any([isinstance(n.op, tcn.blas.GpuDot22Scalar) for n in t])
+#        assert any([isinstance(n.op, tcn.basic_ops.GpuAllocEmpty)
+#                    for n in t])
+        assert numpy.allclose(f(av, bv, 0.5), f2(av, bv, 0.5))
+
+        f = theano.function([a, b, scalar],
+                            tensor.blas._dot22scalar(a, b, scalar),
+                            mode=mode_with_gpu)
         f2 = theano.function([a, b, scalar], tensor.dot(a, b) * scalar)
         t = f.maker.fgraph.toposort()
         assert len(t) == 4
@@ -108,7 +202,6 @@ def test_dot22scalar():
         assert isinstance(t[2].op, tcn.blas.GpuDot22Scalar)
         assert isinstance(t[3].op, tcn.HostFromGpu)
         assert numpy.allclose(f(av, bv, 0.5), f2(av, bv, 0.5))
-
     cmp((3, 4), (4, 5))
     cmp((0, 4), (4, 5))
     cmp((3, 4), (4, 0))
@@ -203,26 +296,26 @@ class TestBlasStridesGpu(TestBlasStrides):
 
 if 0:
     # This is commented out because it doesn't make sense...
-    # tcn.blas has no op called DownsampleFactorMax
+    # tcn.blas has no op called Pool
     # tcn.blas has an op called GpuDownsampleFactorMax, but that op requires arguments that are
     # CudaNdarrayType variables... so rethink this test?
     def test_maxpool():
         """TODO: test the gpu version!!! """
-        for d0, d1, r_true, r_false in [(4,4,[[[[5,7],[13,15]]]],[[[[5,7],[13,15]]]]),
-                                        (5,5,[[[[6, 8],[ 16, 18], [ 21, 23]]]],
-                                         [[[[6, 8, 9],[ 16, 18, 19], [ 21, 23, 24]]]])]:
-            for border,ret in [(True,r_true),(False, r_false)]:
-                ret=numpy.array(ret)
-                a = tcn.blas.DownsampleFactorMax((2,2),border)
+        for d0, d1, r_true, r_false in [(4, 4, [[[[5, 7], [13, 15]]]], [[[[5, 7], [13, 15]]]]),
+                                        (5, 5, [[[[6, 8], [ 16, 18], [ 21, 23]]]],
+                                         [[[[6, 8, 9], [ 16, 18, 19], [ 21, 23, 24]]]])]:
+            for border, ret in [(True, r_true), (False, r_false)]:
+                ret = numpy.array(ret)
+                a = tcn.blas.Pool((2, 2), border)
                 dmatrix4 = tensor.TensorType("float32", (False, False, False, False))
                 b = dmatrix4()
                 f = pfunc([b], [a(b)], mode=mode_with_gpu)
 
-                bval = numpy.arange(0,d0*d1).reshape(1,1,d0,d1)
+                bval = numpy.arange(0, d0*d1).reshape(1, 1, d0, d1)
                 r = f(bval)[0]
     #            print bval, bval.shape, border
-                #print r, r.shape
-                assert (ret==r).all()
+                # print r, r.shape
+                assert (ret == r).all()
 
 
 def test_downsample():
@@ -269,8 +362,8 @@ def test_downsample():
             if float(shp[3]) / ds[1] > 512:
                 continue
             for ignore_border in (True, False):
-                #print 'test_downsample', shp, ds, ignore_border
-                ds_op = DownsampleFactorMax(ds, ignore_border=ignore_border)
+                # print 'test_downsample', shp, ds, ignore_border
+                ds_op = Pool(ds, ignore_border=ignore_border)
 
                 a = tcn.shared_constructor(my_rand(*shp), 'a')
                 f = pfunc([], ds_op(tensor.as_tensor_variable(a)),
@@ -280,7 +373,7 @@ def test_downsample():
                 assert any([isinstance(node.op,
                                        tcn.blas.GpuDownsampleFactorMax)
                     for node in f.maker.fgraph.toposort()])
-                assert any([isinstance(node.op, DownsampleFactorMax)
+                assert any([isinstance(node.op, Pool)
                     for node in f2.maker.fgraph.toposort()])
                 assert numpy.allclose(f(), f2())
 
@@ -305,20 +398,20 @@ def test_downsample():
                 assert any([isinstance(node.op,
                                        tcn.blas.GpuDownsampleFactorMaxGrad)
                             for node in g.maker.fgraph.toposort()])
-                assert any([isinstance(node.op, DownsampleFactorMaxGrad)
+                assert any([isinstance(node.op, PoolGrad)
                             for node in g2.maker.fgraph.toposort()])
                 assert numpy.allclose(g(), g2()), shp
 
                 ggf = gradient.Lop(tensor.grad((ds_op(
                     tensor.as_tensor_variable(a))**2).sum(), a), a, a)
-                
+
                 ref_mode = copy.copy(mode_without_gpu)
                 ref_mode.check_py_code = False
                 gpu_mode = copy.copy(mode_with_gpu)
                 gpu_mode.check_py_code = False
                 gg = pfunc([], ggf, mode=gpu_mode)
                 gg2 = pfunc([], ggf, mode=ref_mode)
-                
+
                 assert any([isinstance(node.op,
                                        tcn.blas.GpuDownsampleFactorMaxGradGrad)
                             for node in gg.maker.fgraph.toposort()])
@@ -369,7 +462,7 @@ class TestGpuGemvNoTransfer(TestCase, BaseGemv,
 
 
 class TestVectorMatrixDot(TestCase):
-    ### Tolerance factor used in this tests
+    # Tolerance factor used in this tests
     atol = 1e-6
     ##########################
 
@@ -380,8 +473,8 @@ class TestVectorMatrixDot(TestCase):
                                        dtype='float32'))
         no_gpu_f = theano.function([], theano.dot(v, m), mode=mode_without_gpu)
         gpu_f = theano.function([], theano.dot(v, m), mode=mode_with_gpu)
-        #gpu_f2 is needed to test the case when the input is not on the gpu
-        #but the output is moved to the gpu.
+        # gpu_f2 is needed to test the case when the input is not on the gpu
+        # but the output is moved to the gpu.
         gpu_f2 = theano.function([], tcn.gpu_from_host(theano.dot(v, m)),
                 mode=mode_with_gpu)
 
@@ -409,8 +502,8 @@ class TestVectorMatrixDot(TestCase):
                                        dtype='float32'))
         no_gpu_f = theano.function([], theano.dot(m, v), mode=mode_without_gpu)
         gpu_f = theano.function([], theano.dot(m, v), mode=mode_with_gpu)
-        #gpu_f2 is needed to test the case when the input is not on the gpu
-        #but the output is moved to the gpu.
+        # gpu_f2 is needed to test the case when the input is not on the gpu
+        # but the output is moved to the gpu.
         gpu_f2 = theano.function([], tcn.gpu_from_host(theano.dot(m, v)),
                 mode=mode_with_gpu)
 
@@ -435,8 +528,8 @@ class TestVectorMatrixDot(TestCase):
         no_gpu_f = theano.function([], v2 + theano.dot(m, v1),
                 mode=mode_without_gpu)
         gpu_f = theano.function([], v2 + theano.dot(m, v1), mode=mode_with_gpu)
-        #gpu_f2 is needed to test the case when the input is not on the gpu
-        #but the output is moved to the gpu.
+        # gpu_f2 is needed to test the case when the input is not on the gpu
+        # but the output is moved to the gpu.
         gpu_f2 = theano.function([], tcn.gpu_from_host(v2 + theano.dot(m, v1)),
                 mode=mode_with_gpu)
 
