@@ -1,7 +1,10 @@
-"""Provide CudaNdarrayType
 """
+Provide CudaNdarrayType.
+
+"""
+from __future__ import print_function
 import os
-import copy_reg
+import six.moves.copyreg as copyreg
 import warnings
 
 import numpy
@@ -10,7 +13,7 @@ import theano
 from theano import Type, Variable
 from theano import tensor, config
 from theano import scalar as scal
-from theano.compat.six import StringIO
+from six import StringIO
 
 try:
     # We must do those import to be able to create the full doc when nvcc
@@ -18,6 +21,16 @@ try:
     import cuda_ndarray.cuda_ndarray as cuda
     from theano.sandbox.cuda.nvcc_compiler import NVCC_compiler
     import cuda_ndarray
+    # Python 3 does not necessarily set __file__. May need manual setting.
+    # The problem is known to occur on Windows 10 with Python 3.4 installed by Anaconda.
+    try:
+        cuda_ndarray.__file__
+    except AttributeError:
+        from theano.gof.cmodule import get_lib_extension
+        # Only works with Python 3, but it's fine, because Python 2
+        # guarantees to set __file__ when importing any module.
+        cuda_ndarray.__file__ = os.path.join(cuda_ndarray.__path__._path[0],
+                                             'cuda_ndarray.' + get_lib_extension())
 except ImportError:
     # Used to know that `cuda` could not be properly imported.
     cuda = None
@@ -30,36 +43,47 @@ class CudaNdarrayType(Type):
     dtype = 'float32'
 
     Variable = None
-    """ This will be set to the Variable type corresponding to this class.
+    """
+    This will be set to the Variable type corresponding to this class.
 
     That variable type is `CudaNdarrayVariable` defined in the
     ``var.py`` file beside this one.
 
-    :note: The var file depends on the file basic_ops.py, which
-    depends on this file.  A cyclic dependency is avoided by not
-    hardcoding ``Variable = CudaNdarrayVariable``.
+    Notes
+    -----
+    The var file depends on the file basic_ops.py, which depends on this file.
+    A cyclic dependency is avoided by not hardcoding
+    ``Variable = CudaNdarrayVariable``.
+
     """
 
     Constant = None
-    """ This will be set to `CudaNdarrayConstant` defined in ``var.py``
+    """
+    This will be set to `CudaNdarrayConstant` defined in ``var.py``.
 
-    :note:
+    Notes
+    -----
     The var file depends on the file basic_ops.py, which depends on this file.
     A cyclic dependency is avoided by not hardcoding this class.
+
     """
 
     SharedVariable = None
-    """ This will be set to `CudaNdarraySharedVariable` defined in ``var.py``
+    """
+    This will be set to `CudaNdarraySharedVariable` defined in ``var.py``.
 
-    :note:
+    Notes
+    -----
     The var file depends on the file basic_ops.py, which depends on this file.
     A cyclic dependency is avoided by not hardcoding this class.
+
     """
 
     if cuda is not None:
         value_zeros = staticmethod(cuda.CudaNdarray.zeros)
     """
-    Create an CudaNdarray full of 0 values
+    Create an CudaNdarray full of 0 values.
+
     """
 
     def __init__(self, broadcastable, name=None, dtype=None):
@@ -67,7 +91,7 @@ class CudaNdarrayType(Type):
             raise TypeError('%s only supports dtype float32 for now. Tried '
                             'using dtype %s for variable %s' %
                             (self.__class__.__name__, dtype, name))
-        self.broadcastable = tuple(broadcastable)
+        self.broadcastable = tuple(bool(b) for b in broadcastable)
         self.name = name
         self.dtype_specs()  # error checking is done there
 
@@ -118,12 +142,14 @@ class CudaNdarrayType(Type):
                         % (self, self.dtype, data, converted_data, self.dtype),
                         data)
 
-    def filter_variable(self, other):
-        """Convert a Variable into a CudaNdarrayType, if compatible.
+    def filter_variable(self, other, allow_convert=True):
+        """
+        Convert a Variable into a CudaNdarrayType, if compatible.
 
         This Variable should either already be a CudaNdarrayType, or be
         a TensorType. It has to have the right number of dimensions,
         broadcastable pattern, and dtype.
+
         """
         if hasattr(other, '_as_CudaNdarrayVariable'):
             other = other._as_CudaNdarrayVariable()
@@ -145,10 +171,17 @@ class CudaNdarrayType(Type):
             raise TypeError('Incompatible number of dimensions.'
                             ' Expected %d, got %d.' % (self.ndim, other.ndim))
         if other.type.broadcastable != self.broadcastable:
-            raise TypeError('Incompatible broadcastable dimensions.'
-                            ' Expected %s, got %s.' %
-                            (str(other.type.broadcastable),
-                             str(self.broadcastable)))
+            if allow_convert:
+                type2 = other.type.clone(broadcastable=self.broadcastable)
+                other2 = type2.convert_variable(other)
+            else:
+                other2 = None
+            if other2 is None:
+                raise TypeError('Incompatible broadcastable dimensions.'
+                                ' Expected %s, got %s.' %
+                                (str(other.type.broadcastable),
+                                 str(self.broadcastable)))
+            other = other2
 
         return theano.sandbox.cuda.basic_ops.GpuFromHost()(other)
 
@@ -156,9 +189,9 @@ class CudaNdarrayType(Type):
     def bound(a):
         high = a.gpudata
         low = a.gpudata
-        #stride is in the number of element.
-        #we must convert that to bytes in case we
-        #will view the element as a different type.
+        # stride is in the number of element.
+        # we must convert that to bytes in case we
+        # will view the element as a different type.
         elem_size = numpy.zeros(0, dtype=a.dtype).dtype.itemsize
 
         for stri, shp in zip(a._strides, a.shape):
@@ -170,8 +203,8 @@ class CudaNdarrayType(Type):
 
     @staticmethod
     def may_share_memory(a, b):
-        #when this is called with a an ndarray and b
-        #a sparce matrix, numpy.may_share_memory fail.
+        # when this is called with a an ndarray and b
+        # a sparce matrix, numpy.may_share_memory fail.
         if a is b:
             return True
         if a.__class__ is b.__class__:
@@ -185,13 +218,13 @@ class CudaNdarrayType(Type):
 
     @staticmethod
     def values_eq(a, b):
-        #TODO: make the comparaison without transfert.
+        # TODO: make the comparaison without transfert.
         return tensor.TensorType.values_eq(numpy.asarray(a), numpy.asarray(b))
 
     @staticmethod
     def values_eq_approx(a, b, allow_remove_inf=False, allow_remove_nan=False,
                          rtol=None, atol=None):
-        #TODO: make the comparaison without transfert.
+        # TODO: make the comparaison without transfert.
         return tensor.TensorType.values_eq_approx(
             numpy.asarray(a),
             numpy.asarray(b),
@@ -201,13 +234,15 @@ class CudaNdarrayType(Type):
         )
 
     def dtype_specs(self):
-        """Return a tuple (python type, c type, numpy typenum) that
-        corresponds to self.dtype.
+        """
+        Return a tuple (python type, c type, numpy typenum) that corresponds
+        to self.dtype.
 
         This function is used internally as part of C code generation.
+
         """
-        #TODO: add more type correspondances for e.g. int32, int64, float32,
-        #complex64, etc.
+        # TODO: add more type correspondances for e.g. int32, int64, float32,
+        # complex64, etc.
         try:
             return {'float32': (float, 'npy_float32', 'NPY_FLOAT32'),
                     'float64': (float, 'npy_float64', 'NPY_FLOAT64'),
@@ -228,7 +263,10 @@ class CudaNdarrayType(Type):
                     self.__class__.__name__, self.dtype))
 
     def __eq__(self, other):
-        """Compare True iff other is the same kind of CudaNdarrayType"""
+        """
+        Compare True iff other is the same kind of CudaNdarrayType.
+
+        """
         return (type(self) == type(other) and
                 other.broadcastable == self.broadcastable)
 
@@ -240,12 +278,16 @@ class CudaNdarrayType(Type):
             return theano.tensor.patternbroadcast(var, self.broadcastable)
 
     def __hash__(self):
-        """Hash equal for same kinds of CudaNdarrayType"""
+        """
+        Hash equal for same kinds of CudaNdarrayType.
+
+        """
         return hash(type(self)) ^ hash(self.broadcastable)
 
     ndim = property(lambda self: len(self.broadcastable),
                     doc="number of dimensions")
-    """Number of dimensions
+    """
+    Number of dimensions.
 
     This read-only property is the preferred way to get the number of
     dimensions of a `CudaNdarrayType`.
@@ -253,12 +295,14 @@ class CudaNdarrayType(Type):
     """
 
     def make_variable(self, name=None):
-        """Return a `TensorVariable` of this type
+        """
+        Return a `TensorVariable` of this type.
 
-        :Parameters:
-         - `name`: str
-           A pretty name to identify this `Variable` when printing and
-           debugging
+        Parameters
+        ----------
+        name : str
+            A pretty name to identify this `Variable` when printing and
+            debugging.
 
         """
         return self.Variable(self, name=name)
@@ -296,7 +340,7 @@ class CudaNdarrayType(Type):
         sio = StringIO()
         fail = sub['fail']
         nd = self.ndim
-        print >> sio, """
+        print("""
         assert(py_%(name)s->ob_refcnt >= 2); // There should be at least one ref from the container object,
         // and one ref from the local scope.
 
@@ -305,9 +349,9 @@ class CudaNdarrayType(Type):
             //fprintf(stderr, "c_extract CNDA object w refcnt %%p %%i\\n", py_%(name)s, (py_%(name)s->ob_refcnt));
             %(name)s = (CudaNdarray*)py_%(name)s;
             //std::cerr << "c_extract " << %(name)s << '\\n';
-        """ % locals()
+        """ % locals(), file=sio)
         if(check_input):
-            print >> sio, """
+            print("""
                 if (%(name)s->nd != %(nd)s)
                 {
                     PyErr_Format(PyExc_RuntimeError,
@@ -317,10 +361,10 @@ class CudaNdarrayType(Type):
                     %(fail)s;
                 }
                 //std::cerr << "c_extract " << %(name)s << " nd check passed\\n";
-            """ % locals()
+            """ % locals(), file=sio)
             for i, b in enumerate(self.broadcastable):
                 if b and check_broadcast:
-                    print >> sio, """
+                    print("""
                 if (CudaNdarray_HOST_DIMS(%(name)s)[%(i)s] != 1)
                 {
                     PyErr_Format(PyExc_RuntimeError,
@@ -342,8 +386,8 @@ class CudaNdarrayType(Type):
                     %(fail)s;
                 }
                 //std::cerr << "c_extract " << %(name)s << "bcast check %(i)s passed\\n";
-                    """ % locals()
-            print >> sio, """
+                    """ % locals(), file=sio)
+            print("""
                 assert(%(name)s);
                 Py_INCREF(py_%(name)s);
             }
@@ -362,18 +406,20 @@ class CudaNdarrayType(Type):
                 %(fail)s;
             }
             //std::cerr << "c_extract done " << %(name)s << '\\n';
-            """ % locals()
+            """ % locals(), file=sio)
         else:
-            print >> sio, """
+            print("""
                 assert(%(name)s);
                 Py_INCREF(py_%(name)s);
             }
-            """ % locals()
-        #print sio.getvalue()
+            """ % locals(), file=sio)
+        # print sio.getvalue()
         return sio.getvalue()
 
     def c_extract_out(self, name, sub, check_input=True, check_broadcast=True):
-        """ To allow the hack to skip check_broadcast.
+        """ 
+        To allow the hack to skip check_broadcast.
+
         """
         return """
         if (py_%(name)s == Py_None)
@@ -403,7 +449,10 @@ class CudaNdarrayType(Type):
         """ % locals()
 
     def c_sync(self, name, sub):
-        """Override `CLinkerOp.c_sync` """
+        """
+        Override `CLinkerOp.c_sync`.
+
+        """
         return """
         //std::cerr << "sync\\n";
         if (NULL == %(name)s) {
@@ -425,11 +474,17 @@ class CudaNdarrayType(Type):
         """ % locals()
 
     def c_headers(self):
-        """Override `CLinkerOp.c_headers` """
+        """
+        Override `CLinkerOp.c_headers`.
+
+        """
         return ['cuda_ndarray.cuh']
 
     def c_header_dirs(self):
-        """Override `CLinkerOp.c_headers` """
+        """
+        Override `CLinkerOp.c_headers`.
+
+        """
         ret = [os.path.dirname(cuda_ndarray.__file__)]
         cuda_root = config.cuda.root
         if cuda_root:
@@ -449,9 +504,9 @@ class CudaNdarrayType(Type):
         return ""
 
     def c_code_cache_version(self):
-        #return ()
-        #no need to put nvcc.fastmath in the tuple as the
-        #c_compile_args is put in the key.
+        # return ()
+        # no need to put nvcc.fastmath in the tuple as the
+        # c_compile_args is put in the key.
         return (3,)  # cublas v2 changes
 
     def c_compiler(self):
@@ -545,7 +600,7 @@ def CudaNdarray_unpickler(npa):
     else:
         raise ImportError("Cuda not found. Cannot unpickle CudaNdarray")
 
-copy_reg.constructor(CudaNdarray_unpickler)
+copyreg.constructor(CudaNdarray_unpickler)
 
 
 def CudaNdarray_pickler(cnda):
@@ -553,5 +608,5 @@ def CudaNdarray_pickler(cnda):
 
 # In case cuda is not imported.
 if cuda is not None:
-    copy_reg.pickle(cuda.CudaNdarray, CudaNdarray_pickler,
+    copyreg.pickle(cuda.CudaNdarray, CudaNdarray_pickler,
                     CudaNdarray_unpickler)
