@@ -1,28 +1,40 @@
+from __future__ import print_function
+
 import numpy as N
+from six.moves import xrange
+
+import theano
 from theano.tensor import basic as T
 from theano.misc import strutil
-import theano
 from theano.gradient import grad_undefined
 from theano.gradient import DisconnectedType
 
 
 class ConvTransp3D(theano.Op):
-    """ "Transpose" of Conv3D (Conv3D implements multiplication by an implicitly defined matrix W. This implements multiplication by its transpose) """
-    def __eq__(self, other):
-        return type(self) == type(other)
+    """
+    "Transpose" of Conv3D (Conv3D implements multiplication by an implicitly
+    defined matrix W. This implements multiplication by its transpose).
 
-    def __hash__(self):
-        return hash(type(self))
+    """
+
+    __props__ = ()
 
     def c_code_cache_version(self):
         return (3,)
 
     def make_node(self, W, b, d, H, RShape=None):
         """
-        :param W: Weights, filter
-        :param b: bias, shape == (W.shape[0],)
-        :param d: strides when moving the filter over the input
-        :param H: The output of Conv3D
+        Parameters
+        ----------
+        W
+            Weights, filter
+        b
+            Bias, shape == (W.shape[0],).
+        d
+            Strides when moving the filter over the input.
+        H
+            The output of Conv3D.
+
         """
         W_ = T.as_tensor_variable(W)
         b_ = T.as_tensor_variable(b)
@@ -33,12 +45,15 @@ class ConvTransp3D(theano.Op):
         else:
             RShape_ = T.as_tensor_variable([-1, -1, -1])
 
-        return theano.Apply(self, inputs=[W_,b_,d_,H_, RShape_], outputs = [ T.TensorType(H_.dtype, (False,False,False,False,False))() ] )
+        return theano.Apply(self,
+                            inputs=[W_, b_, d_, H_, RShape_],
+                            outputs=[T.TensorType(H_.dtype,
+                                     (False, False, False, False, False))()])
 
     def infer_shape(self, node, input_shapes):
         W, b, d, H, RShape = node.inputs
         W_shape, b_shape, d_shape, H_shape, RShape_shape = input_shapes
-        return [(H_shape[0],  RShape[0], RShape[1], RShape[2], W_shape[4])]
+        return [(H_shape[0], RShape[0], RShape[1], RShape[2], W_shape[4])]
 
     def connection_pattern(self, node):
         return [[True], [True], [True], [True], [False]]
@@ -46,9 +61,9 @@ class ConvTransp3D(theano.Op):
     def grad(self, inputs, output_gradients):
         W, b, d, H, RShape = inputs
         dCdR, = output_gradients
-        dCdH = conv3D(dCdR, W, T.zeros_like(H[0, 0, 0, 0, :]), d)
+        dCdH = theano.tensor.nnet.conv3D(dCdR, W, T.zeros_like(H[0, 0, 0, 0, :]), d)
         WShape = W.shape
-        dCdW = convGrad3D(dCdR, d, WShape, H)
+        dCdW = theano.tensor.nnet.convGrad3D(dCdR, d, WShape, H)
         dCdb = T.sum(dCdR, axis=(0, 1, 2, 3))
         # not differentiable, since d affects the output elements
         dCdd = grad_undefined(self, 2, d)
@@ -75,12 +90,13 @@ class ConvTransp3D(theano.Op):
         else:
             b_name = 'anon_b'
 
-
-        dCdW.name = 'ConvTransp3D_dCdW.H='+H_name+',dCdR='+dCdR_name+',W='+W_name
-        dCdb.name = 'ConvTransp3D_dCdb.H='+H_name+',dCdR='+dCdR_name+',W='+W_name+',b='+b_name
+        dCdW.name = ('ConvTransp3D_dCdW.H=' + H_name + ',dCdR=' + dCdR_name +
+                     ',W=' + W_name)
+        dCdb.name = ('ConvTransp3D_dCdb.H=' + H_name + ',dCdR=' + dCdR_name +
+                     ',W=' + W_name + ',b=' + b_name)
         dCdH.name = 'ConvTransp3D_dCdH.H=' + H_name + ',dCdR=' + dCdR_name
 
-        return [dCdW,  dCdb, dCdd, dCdH, dCdRShape]
+        return [dCdW, dCdb, dCdd, dCdH, dCdRShape]
 
     def perform(self, node, inputs, output_storage):
         W, b, d, H, RShape = inputs
@@ -329,7 +345,7 @@ class ConvTransp3D(theano.Op):
 
 convTransp3D = ConvTransp3D()
 
-#If the input size wasn't a multiple of D we may need to cause some automatic padding to get the right size of reconstruction
+# If the input size wasn't a multiple of D we may need to cause some automatic padding to get the right size of reconstruction
 
 
 def computeR(W, b, d, H, Rshape=None):
@@ -338,7 +354,7 @@ def computeR(W, b, d, H, Rshape=None):
     assert len(b.shape) == 1
     assert len(d) == 3
 
-    outputChannels,  filterHeight, filterWidth, filterDur, \
+    outputChannels, filterHeight, filterWidth, filterDur, \
         inputChannels = W.shape
     batchSize, outputHeight, outputWidth, outputDur, \
         outputChannelsAgain = H.shape
@@ -356,29 +372,29 @@ def computeR(W, b, d, H, Rshape=None):
 
     if Rshape is not None and Rshape[0] != -1:
         if Rshape[0] < videoHeight:
-            print (Rshape[0], videoHeight)
+            print((Rshape[0], videoHeight))
             assert False
         assert Rshape[1] >= videoWidth
         assert Rshape[2] >= videoDur
 
-        #print "setting video size to Rshape = "+str(Rshape)
+        # print "setting video size to Rshape = "+str(Rshape)
 
         videoHeight, videoWidth, videoDur = Rshape
-    #else:
+    # else:
     #       print "No Rshape passed in"
 
-    #print "video size: "+str((videoHeight, videoWidth, videoDur))
+    # print "video size: "+str((videoHeight, videoWidth, videoDur))
 
     R = N.zeros((batchSize, videoHeight,
-            videoWidth, videoDur, inputChannels), dtype=H.dtype)
+                videoWidth, videoDur, inputChannels), dtype=H.dtype)
 
-    #R[i,j,r,c,t] = b_j + sum_{rc,rk | d \circ rc + rk = r} sum_{cc,ck | ...} sum_{tc,tk | ...} sum_k W[k, j, rk, ck, tk] * H[i,k,rc,cc,tc]
+    # R[i,j,r,c,t] = b_j + sum_{rc,rk | d \circ rc + rk = r} sum_{cc,ck | ...} sum_{tc,tk | ...} sum_k W[k, j, rk, ck, tk] * H[i,k,rc,cc,tc]
     for i in xrange(0, batchSize):
-        #print '\texample '+str(i+1)+'/'+str(batchSize)
+        # print '\texample '+str(i+1)+'/'+str(batchSize)
         for j in xrange(0, inputChannels):
-            #print '\t\tfeature map '+str(j+1)+'/'+str(inputChannels)
+            # print '\t\tfeature map '+str(j+1)+'/'+str(inputChannels)
             for r in xrange(0, videoHeight):
-                #print '\t\t\trow '+str(r+1)+'/'+str(videoHeight)
+                # print '\t\t\trow '+str(r+1)+'/'+str(videoHeight)
                 for c in xrange(0, videoWidth):
                     for t in xrange(0, videoDur):
                         R[i, r, c, t, j] = b[j]
@@ -407,8 +423,8 @@ def computeR(W, b, d, H, Rshape=None):
                                     if tk < 0:
                                         break
 
-                                    R[
-                                        i,r,c,t,j] += N.dot(W[:,rk,ck,tk,j], H[i,rc,cc,tc,:] )
+                                    R[i, r, c, t, j] += N.dot(
+                                        W[:, rk, ck, tk, j], H[i, rc, cc, tc, :])
 
                                     tc += 1
                                 ""  # close loop over tc
@@ -424,7 +440,3 @@ def computeR(W, b, d, H, Rshape=None):
     ""  # close loop over i
 
     return R
-
-
-from theano.tensor.nnet.Conv3D import conv3D
-from theano.tensor.nnet.ConvGrad3D import convGrad3D
