@@ -1,20 +1,26 @@
 """Define random number Type (`RandomStateType`) and Op (`RandomFunction`)."""
-__docformat__ = "restructuredtext en"
+from __future__ import print_function
+
 import sys
 from copy import copy
 
 import numpy
+from six import string_types
+from six.moves import reduce, xrange
 
-#local imports
+# local imports
 import theano
 from theano import tensor
 from theano.tensor import opt
 from theano import gof
 from theano.compile import optdb
 
+__docformat__ = "restructuredtext en"
+
 
 class RandomStateType(gof.Type):
-    """A Type wrapper for numpy.random.RandomState
+    """
+    A Type wrapper for numpy.random.RandomState.
 
     The reason this exists (and `Generic` doesn't suffice) is that
     RandomState objects that would appear to be equal do not compare
@@ -82,67 +88,57 @@ class RandomStateType(gof.Type):
 
 # Register RandomStateType's C code for ViewOp.
 theano.compile.register_view_op_c_code(
-        RandomStateType,
-        """
-        Py_XDECREF(%(oname)s);
-        %(oname)s = %(iname)s;
-        Py_XINCREF(%(oname)s);
-        """,
-        1)
+    RandomStateType,
+    """
+    Py_XDECREF(%(oname)s);
+    %(oname)s = %(iname)s;
+    Py_XINCREF(%(oname)s);
+    """,
+    1)
 
 random_state_type = RandomStateType()
 
 
 class RandomFunction(gof.Op):
-    """Op that draws random numbers from a numpy.random.RandomState object
+    """
+    Op that draws random numbers from a numpy.random.RandomState object.
+
+    Parameters
+    ----------
+    fn : string or function reference
+        A member function of numpy.random.RandomState. A string will
+        be interpreted as the name of a member function of
+        numpy.random.RandomState.
+        Technically, any function with a signature like the ones in
+        numpy.random.RandomState will do. This function must accept
+        the shape (sometimes called size) of the output as the last
+        positional argument.
+    outtype
+        The theano Type of the output.
+    args
+        A list of default arguments for the function
+        kwargs
+        If the 'inplace' key is there, its value will be used to
+        determine if the op operates inplace or not.
+        If the 'ndim_added' key is there, its value indicates how
+        many more dimensions this op will add to the output, in
+        addition to the shape's dimensions (used in multinomial and
+        permutation).
 
     """
 
+    __props__ = ("fn", "outtype", "inplace", "ndim_added")
+
     def __init__(self, fn, outtype, inplace=False, ndim_added=0):
-        """
-        :param fn: a member function of numpy.random.RandomState
-        Technically, any function with a signature like the ones in
-        numpy.random.RandomState will do.  This function must accept
-        the shape (sometimes called size) of the output as the last
-        positional argument.
-
-        :type fn: string or function reference.  A string will
-        be interpreted as the name of a member function of
-        numpy.random.RandomState.
-
-        :param outtype: the theano Type of the output
-
-        :param args: a list of default arguments for the function
-
-        :param kwargs:
-            If the 'inplace' key is there, its value will be used to
-            determine if the op operates inplace or not.
-            If the 'ndim_added' key is there, its value indicates how
-            many more dimensions this op will add to the output, in
-            addition to the shape's dimensions (used in multinomial and
-            permutation).
-        """
         self.__setstate__([fn, outtype, inplace, ndim_added])
 
-    def __eq__(self, other):
-        return type(self) == type(other) \
-            and self.fn == other.fn\
-            and self.outtype == other.outtype\
-            and self.inplace == other.inplace\
-            and self.ndim_added == other.ndim_added
-
-    def __hash__(self):
-        return hash(type(self)) ^ hash(self.fn) \
-                ^ hash(self.outtype)  \
-                ^ hash(self.inplace) ^ hash(self.ndim_added)
-
     def __getstate__(self):
         return self.state
 
     def __setstate__(self, state):
         self.state = state
         fn, outtype, inplace, ndim_added = state
-        if isinstance(fn, basestring):
+        if isinstance(fn, string_types):
             self.fn = getattr(numpy.random.RandomState, fn)
         else:
             self.fn = fn
@@ -157,41 +153,44 @@ class RandomFunction(gof.Op):
 
     def make_node(self, r, shape, *args):
         """
-        :param r: a numpy.random.RandomState instance, or a Variable of Type
-        RandomStateType that will contain a RandomState instance.
+        Parameters
+        ----------
+        r
+            A numpy.random.RandomState instance, or a Variable of Type
+            RandomStateType that will contain a RandomState instance.
+        shape
+            An lvector with a shape defining how many samples
+            to draw.  In the case of scalar distributions, it is the shape
+            of the tensor output by this Op.  In that case, at runtime, the
+            value associated with this lvector must have a length equal to
+            the number of dimensions promised by `self.outtype`.
+            In a more general case, the number of output dimensions,
+            len(self.outtype), is equal to len(shape)+self.ndim_added.
+            The special case where len(shape) == 0 means that the smallest
+            shape compatible with the argument's shape will be used.
+        args
+            The values associated with these variables will be passed to the
+            RandomState function during perform as extra "*args"-style
+            arguments. These should be castable to variables of Type TensorType.
 
-        :param shape: an lvector with a shape defining how many samples
-        to draw.  In the case of scalar distributions, it is the shape
-        of the tensor output by this Op.  In that case, at runtime, the
-        value associated with this lvector must have a length equal to
-        the number of dimensions promised by `self.outtype`.
-        In a more general case, the number of output dimensions,
-        len(self.outtype), is equal to len(shape)+self.ndim_added.
-        The special case where len(shape) == 0 means that the smallest
-        shape compatible with the argument's shape will be used.
-
-        :param args: the values associated with these variables will
-        be passed to the RandomState function during perform as extra
-        "*args"-style arguments.  These should be castable to variables
-        of Type TensorType.
-
-        :rtype: Apply
-
-        :return: Apply with two outputs.  The first output is a
-        gof.generic Variable from which to draw further random numbers.
-        The second output is the outtype() instance holding the random
-        draw.
+        Returns
+        -------
+        Apply
+            Apply with two outputs. The first output is a gof.generic Variable
+            from which to draw further random numbers.
+            The second output is the outtype() instance holding the random
+            draw.
 
         """
         shape_ = tensor.as_tensor_variable(shape, ndim=1)
         if shape == ():
-            shape = shape_.astype('int32')
+            shape = shape_.astype('int64')
         else:
             shape = shape_
         assert shape.type.ndim == 1
         assert (shape.type.dtype == 'int64') or (shape.type.dtype == 'int32')
         if not isinstance(r.type, RandomStateType):
-            print >> sys.stderr, 'WARNING: RandomState instances should be in RandomStateType'
+            print('WARNING: RandomState instances should be in RandomStateType', file=sys.stderr)
             if 0:
                 raise TypeError('r must be RandomStateType instance', r)
         # the following doesn't work because we want to ignore the
@@ -200,7 +199,7 @@ class RandomFunction(gof.Op):
 
         # convert args to TensorType instances
         # and append enough None's to match the length of self.args
-        args = map(tensor.as_tensor_variable, args)
+        args = list(map(tensor.as_tensor_variable, args))
 
         return gof.Apply(self,
                          [r, shape] + args,
@@ -209,7 +208,7 @@ class RandomFunction(gof.Op):
     def infer_shape(self, node, i_shapes):
         r, shp = node.inputs[0:2]
 
-        #if shp is a constant array of len 0, then it means 'automatic shape'
+        # if shp is a constant array of len 0, then it means 'automatic shape'
         unknown_shape = len(getattr(shp, 'data', [0, 1, 2])) == 0
 
         # if ndim_added == 0 and shape != () then shape
@@ -218,7 +217,7 @@ class RandomFunction(gof.Op):
         else:
             # if shape == () then it will depend on args
             # if ndim_added != 0 and shape != () then it will depend on args
-            #Use the default infer_shape implementation.
+            # Use the default infer_shape implementation.
             raise tensor.ShapeError()
 
         return [None, [sample_shp[i] for i in xrange(node.outputs[1].ndim)]]
@@ -230,7 +229,6 @@ class RandomFunction(gof.Op):
         # copy of r if self.inplace is False
         r, shape, args = inputs[0], inputs[1], inputs[2:]
         assert type(r) == numpy.random.RandomState, (type(r), r)
-        r_orig = r
 
         # If shape == [], that means no shape is enforced, and numpy is
         # trusted to draw the appropriate number of samples, numpy uses
@@ -242,16 +240,16 @@ class RandomFunction(gof.Op):
             shape = tuple(shape)
 
         if (shape is not None and
-            self.outtype.ndim != len(shape) + self.ndim_added):
+                self.outtype.ndim != len(shape) + self.ndim_added):
             raise ValueError('Shape mismatch: self.outtype.ndim (%i) !='
                              ' len(shape) (%i) + self.ndim_added (%i)'
-                            % (self.outtype.ndim, len(shape), self.ndim_added))
+                             % (self.outtype.ndim, len(shape), self.ndim_added))
         if not self.inplace:
             r = copy(r)
         rout[0] = r
         rval = self.fn(r, *(args + [shape]))
-        if not isinstance(rval, numpy.ndarray) \
-               or str(rval.dtype) != node.outputs[1].type.dtype:
+        if (not isinstance(rval, numpy.ndarray) or
+                str(rval.dtype) != node.outputs[1].type.dtype):
             rval = theano._asarray(rval, dtype=node.outputs[1].type.dtype)
 
         # When shape is None, numpy has a tendency to unexpectedly
@@ -285,7 +283,7 @@ class RandomFunction(gof.Op):
 
     def grad(self, inputs, outputs):
         return [theano.gradient.grad_undefined(self, k, inp,
-                        'No gradient defined through raw random numbers op')
+                'No gradient defined through raw random numbers op')
                 for k, inp in enumerate(inputs)]
 
     def R_op(self, inputs, eval_points):
@@ -296,12 +294,15 @@ def _infer_ndim_bcast(ndim, shape, *args):
     """
     Infer the number of dimensions from the shape or the other arguments.
 
-    :rtype: (int, variable, tuple) triple, where the variable is an integer
-    vector, and the tuple contains Booleans.
-    :returns: the first element returned is the inferred number of dimensions.
-    The second element is the shape inferred (combining symbolic and constant
-    informations from shape and args).
-    The third element is a broadcasting pattern corresponding to that shape.
+    Returns
+    -------
+    (int, variable, tuple) triple, where the variable is an integer vector,
+    and the tuple contains Booleans
+        The first element returned is the inferred number of dimensions.
+        The second element is the shape inferred (combining symbolic and
+        constant informations from shape and args).
+        The third element is a broadcasting pattern corresponding to that shape.
+
     """
 
     # Find the minimum value of ndim required by the *args
@@ -322,8 +323,8 @@ def _infer_ndim_bcast(ndim, shape, *args):
         else:
             if shape_ndim != ndim:
                 raise ValueError('ndim should be equal to len(shape), but\n',
-                            'ndim = %s, len(shape) = %s, shape = %s'
-                            % (ndim, shape_ndim, shape))
+                                 'ndim = %s, len(shape) = %s, shape = %s'
+                                 % (ndim, shape_ndim, shape))
 
         bcast = []
         pre_v_shape = []
@@ -350,7 +351,8 @@ def _infer_ndim_bcast(ndim, shape, *args):
                                 break
                     else:
                         if n_a_i == 0:
-                            raise ValueError(('Auto-shape of -1 must overlap'
+                            raise ValueError((
+                                'Auto-shape of -1 must overlap'
                                 'with the shape of one of the broadcastable'
                                 'inputs'))
                         else:
@@ -361,26 +363,34 @@ def _infer_ndim_bcast(ndim, shape, *args):
         # post-condition: shape may still contain both symbolic and
         # non-symbolic things
         if len(pre_v_shape) == 0:
-            v_shape = tensor.constant([], dtype='int32')
+            v_shape = tensor.constant([], dtype='int64')
         else:
-            v_shape = tensor.stack(*pre_v_shape)
+            v_shape = tensor.stack(pre_v_shape)
 
     elif shape is None:
         # The number of drawn samples will be determined automatically,
         # but we need to know ndim
         if not args:
             raise TypeError(('_infer_ndim_bcast cannot infer shape without'
-                ' either shape or args'))
+                             ' either shape or args'))
         template = reduce(lambda a, b: a + b, args)
         v_shape = template.shape
         bcast = template.broadcastable
         ndim = template.ndim
     else:
         v_shape = tensor.as_tensor_variable(shape)
+        if v_shape.ndim != 1:
+            raise TypeError(
+                "shape must be a vector or list of scalar, got '%s'" % v_shape)
+
         if ndim is None:
             ndim = tensor.get_vector_length(v_shape)
         bcast = [False] * ndim
 
+    if v_shape.ndim != 1:
+        raise TypeError("shape must be a vector or list of scalar, got '%s'" %
+                        v_shape)
+
     if (not (v_shape.dtype.startswith('int') or
              v_shape.dtype.startswith('uint'))):
         raise TypeError('shape must be an integer vector or list',
@@ -392,11 +402,11 @@ def _infer_ndim_bcast(ndim, shape, *args):
             (ndim, args_ndim), args)
 
     assert ndim == len(bcast)
-    return ndim, tensor.cast(v_shape, 'int32'), tuple(bcast)
+    return ndim, tensor.cast(v_shape, 'int64'), tuple(bcast)
 
 
 def _generate_broadcasting_indices(out_shape, *shapes):
-    '''
+    """
     Return indices over each shape that broadcast them to match out_shape.
 
     The first returned list is equivalent to numpy.ndindex(out_shape),
@@ -406,7 +416,8 @@ def _generate_broadcasting_indices(out_shape, *shapes):
 
     The shapes should have the same length as out_shape. If they are longer,
     the right-most dimensions are ignored.
-    '''
+
+    """
     all_shapes = (out_shape,) + shapes
     # Will contain the return value: a list of indices for each argument
     ret_indices = [[()] for shape in all_shapes]
@@ -415,7 +426,7 @@ def _generate_broadcasting_indices(out_shape, *shapes):
         # Temporary list to generate the indices
         _ret_indices = [[] for shape in all_shapes]
 
-        out_range = range(out_shape[dim])
+        out_range = list(range(out_shape[dim]))
 
         # Verify the shapes are compatible along that dimension
         # and generate the appropriate range: out_range, or [0, ..., 0]
@@ -453,6 +464,7 @@ def uniform(random_state, size=None, low=0.0, high=1.0, ndim=None, dtype=None):
 
     If dtype is not specified, it will be inferred from the dtype of
     low and high, but will be at least as precise as floatX.
+
     """
     low = tensor.as_tensor_variable(low)
     high = tensor.as_tensor_variable(high)
@@ -460,7 +472,7 @@ def uniform(random_state, size=None, low=0.0, high=1.0, ndim=None, dtype=None):
         dtype = tensor.scal.upcast(theano.config.floatX, low.dtype, high.dtype)
     ndim, size, bcast = _infer_ndim_bcast(ndim, size, low, high)
     op = RandomFunction('uniform',
-            tensor.TensorType(dtype=dtype, broadcastable=bcast))
+                        tensor.TensorType(dtype=dtype, broadcastable=bcast))
     return op(random_state, size, low, high)
 
 
@@ -477,6 +489,7 @@ def normal(random_state, size=None, avg=0.0, std=1.0, ndim=None, dtype=None):
 
     If dtype is not specified, it will be inferred from the dtype of
     avg and std, but will be at least as precise as floatX.
+
     """
     avg = tensor.as_tensor_variable(avg)
     std = tensor.as_tensor_variable(std)
@@ -484,7 +497,7 @@ def normal(random_state, size=None, avg=0.0, std=1.0, ndim=None, dtype=None):
         dtype = tensor.scal.upcast(theano.config.floatX, avg.dtype, std.dtype)
     ndim, size, bcast = _infer_ndim_bcast(ndim, size, avg, std)
     op = RandomFunction('normal',
-            tensor.TensorType(dtype=dtype, broadcastable=bcast))
+                        tensor.TensorType(dtype=dtype, broadcastable=bcast))
     return op(random_state, size, avg, std)
 
 
@@ -499,32 +512,34 @@ def binomial(random_state, size=None, n=1, p=0.5, ndim=None,
 
     If size is None, the output shape will be determined by the shapes
     of n and prob.
+
     """
     if prob is not None:
         p = prob
-        print >> sys.stderr, "DEPRECATION WARNING: the parameter prob to the binomal fct have been renamed to p to have the same name as numpy."
+        print("DEPRECATION WARNING: the parameter prob to the binomal fct have been renamed to p to have the same name as numpy.", file=sys.stderr)
     n = tensor.as_tensor_variable(n)
     p = tensor.as_tensor_variable(p)
     ndim, size, bcast = _infer_ndim_bcast(ndim, size, n, p)
     if n.dtype == 'int64':
-        ### THIS WORKS AROUND A NUMPY BUG on 32bit machine
-        ###  Erase when the following works on a 32bit machine:
-        ###  numpy.random.binomial(
-        #          n=numpy.asarray([2,3,4], dtype='int64'),
-        #          p=numpy.asarray([.1, .2, .3], dtype='float64'))
-        n = tensor.cast(n, 'int32')
+        try:
+            numpy.random.binomial(n=numpy.asarray([2, 3, 4], dtype='int64'), p=numpy.asarray([.1, .2, .3], dtype='float64'))
+        except TypeError:
+            # THIS WORKS AROUND A NUMPY BUG on 32bit machine
+            n = tensor.cast(n, 'int32')
     op = RandomFunction('binomial',
-            tensor.TensorType(dtype=dtype, broadcastable=(False,) * ndim))
+                        tensor.TensorType(dtype=dtype,
+                                          broadcastable=(False,) * ndim))
     return op(random_state, size, n, p)
 
 
 def random_integers_helper(random_state, low, high, size):
-    '''
+    """
     Helper function to draw random integers.
 
     This is a generalization of numpy.random.random_integers to the case where
     low and high are tensors.
-    '''
+
+    """
     # Figure out the output shape
     if size is not None:
         out_ndim = len(size)
@@ -575,21 +590,24 @@ def random_integers(random_state, size=None, low=0, high=1, ndim=None,
 
     If size is None, the output shape will be determined by the shapes
     of low and high.
+
     """
     low = tensor.as_tensor_variable(low)
     high = tensor.as_tensor_variable(high)
     ndim, size, bcast = _infer_ndim_bcast(ndim, size, low, high)
     op = RandomFunction(random_integers_helper,
-            tensor.TensorType(dtype=dtype, broadcastable=bcast))
+                        tensor.TensorType(dtype=dtype, broadcastable=bcast))
     return op(random_state, size, low, high)
 
 
 def choice_helper(random_state, a, replace, p, size):
-    """Helper function to draw random numbers using numpy's choice function.
+    """
+    Helper function to draw random numbers using numpy's choice function.
 
     This is a generalization of numpy.random.choice that coerces
     `replace` to a bool and replaces `p` with None when p is a vector
     of 0 elements.
+
     """
     if a.ndim > 1:
         raise ValueError('a.ndim (%i) must be 0 or 1' % a.ndim)
@@ -613,6 +631,7 @@ def choice(random_state, size=None, a=2, replace=True, p=None, ndim=None,
     may be a plain integer to supplement the missing information.
 
     If size is None, a scalar will be returned.
+
     """
     # numpy.random.choice is only available for numpy versions >= 1.7
     major, minor, _ = numpy.version.short_version.split('.')
@@ -631,24 +650,29 @@ def choice(random_state, size=None, a=2, replace=True, p=None, ndim=None,
                                                          broadcastable=bcast))
     return op(random_state, size, a, replace, p)
 
+
 def poisson(random_state, size=None, lam=1.0, ndim=None, dtype='int64'):
     """
     Draw samples from a Poisson distribution.
 
-    The Poisson distribution is the limit of the Binomial distribution for large N.
-    
-    :param lam: float or ndarray-like of the same shape as size parameter
+    The Poisson distribution is the limit of the Binomial distribution for
+    large N.
+
+    Parameters
+    ----------
+    lam : float or ndarray-like of the same shape as size parameter
         Expectation of interval, should be >= 0.
+    size: int or tuple of ints, optional
+        Output shape. If the given shape is, e.g., (m, n, k), then m * n * k
+        samples are drawn.
+    dtype
+        The dtype of the return value (which will represent counts).
 
-    :param size: int or tuple of ints, optional
-        Output shape. If the given shape is, e.g., (m, n, k), then m * n * k samples are drawn.
+    size or ndim must be given.
 
-    :param dtype: the dtype of the return value (which will represent counts)
-
-    size or ndim must be given
     """
     lam = tensor.as_tensor_variable(lam)
-    
+
     ndim, size, bcast = _infer_ndim_bcast(ndim, size)
 
     op = RandomFunction("poisson", tensor.TensorType(dtype=dtype,
@@ -657,7 +681,8 @@ def poisson(random_state, size=None, lam=1.0, ndim=None, dtype='int64'):
 
 
 def permutation_helper(random_state, n, shape):
-    """Helper function to generate permutations from integers.
+    """
+    Helper function to generate permutations from integers.
 
     permutation_helper(random_state, n, (1,)) will generate a permutation of
     integers 0..n-1.
@@ -670,6 +695,7 @@ def permutation_helper(random_state, n, shape):
 
     This is a generalization of numpy.random.permutation to tensors.
     Otherwise it behaves the same.
+
     """
     # n should be a 0-dimension array
     assert n.shape == ()
@@ -686,23 +712,26 @@ def permutation_helper(random_state, n, shape):
     for i in numpy.ndindex(*shape):
         out[i] = random_state.permutation(n)
 
-    #print 'RETURNING', out.shape
+    # print 'RETURNING', out.shape
     return out
 
 
 def permutation(random_state, size=None, n=1, ndim=None, dtype='int64'):
     """
-    Returns permutations of the integers between 0 and n-1, as many times
-    as required by size. For instance, if size=(p,q), p*q permutations
-    will be generated, and the output shape will be (p,q,n), because each
-    permutation is of size n.
+    Return permutations of the integers between 0 and n-1.
+
+    Returns them as many times as required by size. For instance, if size=(p,q),
+    p*q permutations will be generated, and the output shape will be (p,q,n),
+    because each permutation is of size n.
 
     Theano tries to infer the number of dimensions from the length of
     the size argument and the shape of n, but you may always specify it
     with the `ndim` parameter.
 
-    :note:
-        Note that the output will then be of dimension ndim+1.
+    Notes
+    -----
+    Note that the output will then be of dimension ndim+1.
+
     """
     if size is None or size == ():
         if not(ndim is None or ndim == 1):
@@ -713,20 +742,22 @@ def permutation(random_state, size=None, n=1, ndim=None, dtype='int64'):
         bcast = ()
     else:
         ndim, size, bcast = _infer_ndim_bcast(ndim, size)
-    #print "NDIM", ndim, size
+    # print "NDIM", ndim, size
     op = RandomFunction(permutation_helper,
-            tensor.TensorType(dtype=dtype, broadcastable=bcast + (False,)),
-            ndim_added=1)
+                        tensor.TensorType(dtype=dtype,
+                                          broadcastable=bcast + (False,)),
+                        ndim_added=1)
     return op(random_state, size, n)
 
 
 def multinomial_helper(random_state, n, pvals, size):
-    '''
+    """
     Helper function drawing from multinomial distributions.
 
     This is a generalization of numpy.random.multinomial to the case where
     n and pvals are tensors.
-    '''
+
+    """
     # Figure out the shape if it's None
     # Note: the output ndim will be ndim+1, because the multinomial
     # adds a dimension. The length of that dimension is pvals.shape[-1].
@@ -734,14 +765,11 @@ def multinomial_helper(random_state, n, pvals, size):
         ndim = len(size)
     else:
         ndim = max(n.ndim, pvals.ndim - 1)
-    out_ndim = ndim + 1
 
     # broadcast n to ndim dimensions and pvals to ndim+1
     if n.ndim > ndim:
-        raise ValueError(
-            'n.ndim (%i) should not be larger than len(size) (%i)'
-            % (n.ndim, ndim),
-                n, size)
+        raise ValueError('n.ndim (%i) should not be larger than len(size) (%i)'
+                         % (n.ndim, ndim), n, size)
     if n.ndim < ndim:
         n = n.reshape((1,) * (ndim - n.ndim) + n.shape)
 
@@ -784,7 +812,7 @@ def multinomial_helper(random_state, n, pvals, size):
             # because mtrand.pyx has a ValueError that will trigger if
             # sum(pvals[:-1]) > 1.0
             pvi = pvi * (1.0 - 5e-5)
-            #pvi = pvi * .9
+            # pvi = pvi * .9
             pisum = numpy.sum(pvi)
         elif pvi[-1] < 5e-5:  # will this even work?
             pvi = pvi * (1.0 - 5e-5)
@@ -797,31 +825,40 @@ def multinomial_helper(random_state, n, pvals, size):
 
 def multinomial(random_state, size=None, n=1, pvals=[0.5, 0.5],
                 ndim=None, dtype='int64'):
-    """Sample from one or more multinomial distributions defined by
+    """
+    Sample from one or more multinomial distributions defined by
     one-dimensional slices in pvals.
 
-    :param pvals: a tensor of shape "nmulti+(L,)" describing each multinomial
+    Parameters
+    ----------
+    pvals
+        A tensor of shape "nmulti+(L,)" describing each multinomial
         distribution.  This tensor must have the property that
         numpy.allclose(pvals.sum(axis=-1), 1) is true.
-
-    :param size: a vector of shape information for the output; this can also
+    size
+        A vector of shape information for the output; this can also
         specify the "nmulti" part of pvals' shape.  A -1 in the k'th position
         from the right means to borrow the k'th position from the
         right in nmulti. (See examples below.)
         Default ``None`` means size=nmulti.
-
-    :param n: the number of experiments to simulate for each
+    n
+        The number of experiments to simulate for each
         multinomial. This can be a scalar, or tensor, it will be
         broadcasted to have shape "nmulti".
+    dtype
+        The dtype of the return value (which will represent counts)
 
-    :param dtype: the dtype of the return value (which will represent counts)
-
-    :returns: tensor of len(size)+1 dimensions, and shape[-1]==L, with
-        the specified ``dtype``, with the experiment counts.  See
+    Returns
+    -------
+    tensor
+        Tensor of len(size)+1 dimensions, and shape[-1]==L, with
+        the specified ``dtype``, with the experiment counts. See
         examples to understand the shape of the return value, which is
-        derived from both size and pvals.shape.  In return value rval,
+        derived from both size and pvals.shape. In return value rval,
         "numpy.allclose(rval.sum(axis=-1), n)" will be true.
 
+    Extended Summary
+    ----------------
     For example, to simulate n experiments from each multinomial in a batch of
     size B:
 
@@ -855,8 +892,9 @@ def multinomial(random_state, size=None, n=1, pvals=[0.5, 0.5],
     ndim, size, bcast = _infer_ndim_bcast(ndim, size, n, tmp)
     bcast = bcast + (pvals.type.broadcastable[-1],)
     op = RandomFunction(multinomial_helper,
-            tensor.TensorType(dtype=dtype, broadcastable=bcast),
-            ndim_added=1)
+                        tensor.TensorType(dtype=dtype,
+                                          broadcastable=bcast),
+                        ndim_added=1)
     return op(random_state, size, n, pvals)
 
 
@@ -864,8 +902,11 @@ def multinomial(random_state, size=None, n=1, pvals=[0.5, 0.5],
 def random_make_inplace(node):
     op = node.op
     if isinstance(op, RandomFunction) and not op.inplace:
-        new_op = RandomFunction(op.fn, op.outtype, inplace=True,
-                                ndim_added=op.ndim_added)
+        # Read op_fn from op.state, not from op.fn, since op.fn
+        # may not be picklable.
+        op_fn, op_outtype, op_inplace, op_ndim_added = op.__getstate__()
+        new_op = RandomFunction(op_fn, op_outtype, inplace=True,
+                                ndim_added=op_ndim_added)
         return new_op.make_node(*node.inputs).outputs
     return False
 
@@ -883,12 +924,12 @@ class RandomStreamsBase(object):
         return the number of successes.
 
         If the size argument is ambiguous on the number of dimensions,
-        ndim may be a plain integer to supplement the missing
-        information.
+        ndim may be a plain integer to supplement the missing information.
+
         """
         if prob is not None:
             p = prob
-            print >> sys.stderr, "DEPRECATION WARNING: the parameter prob to the binomal fct have been renamed to p to have the same name as numpy."
+            print("DEPRECATION WARNING: the parameter prob to the binomal fct have been renamed to p to have the same name as numpy.", file=sys.stderr)
         return self.gen(binomial, size, n, p, ndim=ndim, dtype=dtype)
 
     def uniform(self, size=None, low=0.0, high=1.0, ndim=None, dtype=None):
@@ -897,8 +938,8 @@ class RandomStreamsBase(object):
         distribution between low and high.
 
         If the size argument is ambiguous on the number of dimensions,
-        ndim may be a plain integer to supplement the missing
-        information.
+        ndim may be a plain integer to supplement the missing information.
+
         """
         return self.gen(uniform, size, low, high, ndim=ndim, dtype=dtype)
 
@@ -908,8 +949,8 @@ class RandomStreamsBase(object):
         the specified standard deviation (std).
 
         If the size argument is ambiguous on the number of dimensions,
-        ndim may be a plain integer to supplement the missing
-        information.
+        ndim may be a plain integer to supplement the missing information.
+
         """
         return self.gen(normal, size, avg, std, ndim=ndim, dtype=dtype)
 
@@ -919,50 +960,56 @@ class RandomStreamsBase(object):
         Sample a random integer between low and high, both inclusive.
 
         If the size argument is ambiguous on the number of dimensions,
-        ndim may be a plain integer to supplement the missing
-        information.
+        ndim may be a plain integer to supplement the missing information.
+
         """
         return self.gen(random_integers, size, low, high, ndim=ndim,
                         dtype=dtype)
-    
+
     def choice(self, size=None, a=2, replace=True, p=None, ndim=None,
                dtype='int64'):
         """
-        Choose values from `a` with or without replacement. `a` can be a 1-D
-        array or a positive scalar. If `a` is a scalar, the samples are drawn
-        from the range 0,...,a-1.
+        Choose values from `a` with or without replacement.
+
+        `a` can be a 1-D array or a positive scalar.
+        If `a` is a scalar, the samples are drawn from the range 0,...,a-1.
 
         If the size argument is ambiguous on the number of dimensions,
-        ndim may be a plain integer to supplement the missing
-        information.
+        ndim may be a plain integer to supplement the missing information.
+
         """
         return self.gen(choice, size, a, replace, p, ndim=ndim, dtype=dtype)
 
     def poisson(self, size=None, lam=None, ndim=None, dtype='int64'):
         """
         Draw samples from a Poisson distribution.
-  
-        The Poisson distribution is the limit of the Binomial distribution for large N.
+
+        The Poisson distribution is the limit of the Binomial distribution for
+        large N.
 
         If the size argument is ambiguous on the number of dimensions,
-        ndim may be a plain integer to supplement the missing
-        information.
+        ndim may be a plain integer to supplement the missing information.
+
         """
         return self.gen(poisson, size, lam, ndim=ndim, dtype=dtype)
 
     def permutation(self, size=None, n=1, ndim=None, dtype='int64'):
         """
-        Returns permutations of the integers between 0 and n-1, as many times
-        as required by size. For instance, if size=(p,q), p*q permutations
-        will be generated, and the output shape will be (p,q,n), because each
+        Return permutations of the integers between 0 and n-1.
+
+        Returns them as many times as required by size. For instance,
+        if size=(p,q), p*q permutations will be generated,
+        and the output shape will be (p,q,n), because each
         permutation is of size n.
 
         Theano tries to infer the number of dimensions from the length
         of the size argument and the shape of n, but you may always
         specify it with the `ndim` parameter.
 
-        .. note::
-            Note that the output will then be of dimension ndim+1.
+        Notes
+        -----
+        Note that the output will then be of dimension ndim+1.
+
         """
         return self.gen(permutation, size, n, ndim=ndim, dtype=dtype)
 
@@ -978,16 +1025,20 @@ class RandomStreamsBase(object):
         of the size argument and the shapes of n and pvals, but you may
         always specify it with the `ndim` parameter.
 
-        .. note::
-            Note that the output will then be of dimension ndim+1.
+        Notes
+        -----
+        Note that the output will then be of dimension ndim+1.
+
         """
         return self.gen(multinomial, size, n, pvals, ndim=ndim, dtype=dtype)
 
     def shuffle_row_elements(self, input):
-        """Return a variable with every row (rightmost index) shuffled.
+        """
+        Return a variable with every row (rightmost index) shuffled.
 
         This uses permutation random variable internally, available via
         the ``.permutation`` attribute of the return value.
+
         """
         perm = self.permutation(size=input.shape[:-1], n=input.shape[-1],
                                 ndim=input.ndim - 1)
