@@ -29,11 +29,15 @@ def test_none_Constant():
     assert o2.equals(NoneConst)
 
     # This trigger equals that returned the wrong answer in the past.
-    import cPickle
+    import six.moves.cPickle as pickle
     import theano
     from theano import tensor
 
     x = tensor.vector('x')
     y = tensor.argmax(x)
-    f = theano.function([x], [y])
-    cPickle.loads(cPickle.dumps(f))
+    kwargs = {}
+    # We can't pickle DebugMode
+    if theano.config.mode in ["DebugMode", "DEBUG_MODE"]:
+        kwargs = {'mode': 'FAST_RUN'}
+    f = theano.function([x], [y], **kwargs)
+    pickle.loads(pickle.dumps(f))
